#ifndef STEVO_H
#define STEVO_H

#include <fstream>
#include <sstream>
#include <vector>

#include <utilities.h>

#include <types.h> //for type "star"

#ifndef SEVN
#include <CL/cl_platform.h> //for cl_int8
#endif

using namespace std;

class Stellarevolution : public utilities
{
	public:
		Stellarevolution(){elements_per_line = 0;
								 line_number = 0;
								 Z_file = "./input/Z_tables.dat";}



		void get_final_parameters(star *s);
        
        int get_phase_BSE(star *s);
        
        
        void get_new_phase(star *s, int k1, int k2, int k3, int k4, int k5, int k6, int k7, int k8,
    float ttarg1, float ttarg2, float ttarg3, float ttarg4);
        void get_Ks(float life_percentage, float t0, float tf, int i0, int i1, int *k1, int *k2, float *ttarg);
        float get_new_value(float t1, float m1, float tt1, float mm1, float ttarget1,
                           float t2, float m2, float tt2, float mm2, float ttarget2,
                           float t3, float m3, float tt3, float mm3, float ttarget3,
                           float t4, float m4, float tt4, float mm4, float ttarget4, 
                           float w1, float w2, float z1, float z2);

		void load_available_metallicities();
		void load_stevo_arrays();
		void read_mass_list(int Z); //read mass list for metallicity index Z
		void load_mass_files();
		void load_CO_files();
		void load_He_files();
		void load_luminosity_files();
		void load_radius_files();
		void load_phase_files();
		void load_time_files();

		void stevoevolve(star *s);

		char* phase_to_char(int int_phase);
		int phase_to_int(char* char_phase);


		void set_remnant_time(float *SN_time);
		void set_weights(cl_float2 *cpu_mass_weights, cl_float2 *cpu_metal_weights);
		void set_array_position(cl_int8 *cpu_stevo_ind, cl_int8 *gmind);
        
        void set_coandhe_times(star *mystar);

		void supernova(star *s);



		//SN explosion functions//////////////////
		void pair_instability(star *s);
		void delayed_SN_explosion(star *s);
		void rapid_SN_explosion(star *s);
		void startrack_SN_explosion(star *s);
		//////////////////////////////////////////
		void set_birth_kick(star *s);
		//////////////////////////////////////////
		float maxwellian_cdf(float x, float sigma);
		float maxwellian_pdf(float x, float sigma);
		//////////////////////////////////////////


		//functions to estimate stellar lifetime///////////////
		double get_base_agb_lum();
		double get_ms_time();
		double get_hg_time();
		double get_base_gb_lum();
		double get_base_ms_lum();
		double get_giant_lum();
		double get_gb_time();
		double get_he_giant_lum();
		double get_he_time();
		double get_agb_time();
		///////////////////////////////////////////////////////
		double get_life_time();
		double get_life_time(double newzams); //overloaded for external, generic use
		////////////////////////////////////////////////////////


		vector<float> metallicity_values; //how many and which metallicities are available
		vector< vector<float> > list_masses; //it collects all the mass lists
		vector< vector<int> >   global_matrix_istart; //it collects the numbers of the starting elements for each mass 


		vector<float>  stevo_masses;
		vector<float>  stevo_times;
		vector<float>  stevo_co;
		vector<float>  stevo_he;
		vector<float>  stevo_radius;
		vector<float>  stevo_luminosity;
		vector<int>    stevo_phase;




	private:

		float m; //mass of the star (input parameter)
		float Z; //metallicity of the star (input parameter)
		cl_int4 final_ind; //array indexes to determine final parameters of stars
		string SN_model; //SN model of each star... it could be different from star to star.. it is individual!!
		float alpha, beta, gamma, delta; //coefficients for SN explosion.
		float birth_kick; //module of the birth kick for compact remnants.
		//************************************************************
		double main_sequence_time; //evolutionary times... (PARSEC tables stop at TP-AGB)
		double hertzsprung_gap_time;
		double giant_branch_time;
		double helium_burning_time;
		double agb_time;
		//*************************************************************
		cl_float2 local_mass_weights;
		cl_float2 local_metal_weights;
		///////////////////////////////////////////////////////////////
		int zhigh, zlow, m1_zlow, m1_zhigh, m2_zlow, m2_zhigh;
		//////////////////////////////////////////////////////////////

		int elements_per_line; //count elements in each line
		ifstream in; //generic ifstream for input files
		string line, value; //generic string to collect data in a generic line
		string Z_file; //file used to read the available metallicities;
		istringstream stream; //generic stream to read input lines
		bool          err; //generic errors catching variable
		int line_number; //count number of lines
		int total_count_per_metallicity; //total number of numbers per metallicity
      vector<string> metallicity_paths; //where to find the stellar evolution files


};




#endif
