#ifndef TYPES_H
#define TYPES_H

#include <string>
#include <vector>

#ifndef SEVN
#include <CL/cl.h>
#else
#include <sevnonly_types.h>
#endif

using namespace std;

#ifndef SEVN
struct cl_kernel_struct{
	cl_kernel evaluation;
	cl_kernel reduction;
	cl_kernel predictor_jp;
	cl_kernel predictor_ip;
    cl_kernel corrector;
    cl_kernel post_corrector;
    cl_kernel energy;
    cl_kernel reduction_energy;
    cl_kernel energycorr;
};


#ifdef HERMITE8
typedef struct __attribute__ ((aligned(128)))
{
    cl_float4  vel;
    cl_float4  acc;
    cl_float4  acc1;
    cl_double4 pos;

}gpu_predictor;
#endif

#ifdef HERMITE6
typedef struct
{
    cl_float4  vel;
    cl_float4  acc;
    cl_double4 pos;

}gpu_predictor;
#endif


typedef struct{
   vector<int> particle;
   double step;
   double localtime;
   vector<int> toerase;
}block_struct;
#endif



//stellar evolution parameters
typedef struct __attribute__ ((aligned(512))){	

	float  z; // initial metallicity
	float sntime; //compact remnant time;
	float he_t; //variable to store the helium mass
   float co_t; //variable to store the CO mass
	float r_t; //variable to store radius
	float l_t; //variable to store luminosity
	float T_t; //variable to store temperature
	float abs_Mag_t; //variable to store absolute magnitude
	char *phase; //phase of the star... main sequence, black hole, etc etc
	char *phase_fin;

	cl_float4 ospin; /* star rotation vector 
	 x y z is direction UNITY VECTOR
	 w is the magnitude of rotation in yr^-1 */

	float m_dot; // dm/dt (total) at present time
	float mHE_dot; // dm/dt (Helium) at present time
	float mHE_dot_init; // dm/dt (Helium) at previous time
	float mCO_dot; // dm/dt (Carbin-Oxygen) at present time
	float mwind_accr; //mass accreted due to stellar winds
	
	float mHE_prev; //He core mass at the previous step;
    float time_step_prev; //previous time step;
    
    int countHG;
    bool mHE_dot_set;
	
	float time_step;

	//final values of different variables follow:
	float he_fin;
	float l_fin;
	float r_fin;
   float co_fin;
	////////////////////////////////////

   float co_res;// CO reservoir... to avoid sudden CO removal
	//40 Bytes here
	float fallback; //fallback material (relative amount)
   float Dco; //differences of final COs
	char* sn; //SN explosion model
	
	
	float pisn_correction;
	cl_float2 wm; // weights for masses
   cl_float2 wz; //weights for metallicity
	//+28 Bytes here ----> 68 Bytes total
	
	
	
	double mzams; // initial mass of the star
	double m_t; //variable to store the total mass as a function of time
   double m_fin; //final pre-SN mass of the star
   double m_rem; //remnant mass

   //this tells you the age of your star
	double t_loc; //local table-track time.... stores the difference between track time and physical time
   
	
	
	double t_rejuv; //difference between local/global time
	//+48 Bytes here ---> total 116 Bytes
	cl_float4 vkick; //birth kick
	cl_int8 ind; //indexes to follow tracks in tables
	//+ 48 Bytes here ----> total 164 Bytes

	cl_double4 pos; //positions of the two particles scaled to COM (if the star is in binary)
	cl_double4 vel; //velocities of the two particles scaled to the velocity of COM (if the star is in binary)

	float rcore; //core radius
	float mcore; //core mass... it should be determined from m_t, he_t and co_t
	float menv; //envelope mass... it should be determined from m_t, he_t and co_t
	
	cl_float4 lifetimes; //lifetimes of the stars on neighbour tracks
	cl_float4 neigh_hestart;
    cl_float4 neigh_costart;
    float he_start;
    float co_start;
	
	cl_int8 gmatrixind;
    
    double maccr;
    double pl; //percentage of life in the three different phases (H, He core, CO core)
    bool RLF; //is this star filling its Roche lobe?
    bool isempty; //does this star has been disrupted after merger/collision?

}star;


//struct dedicated to binary stars...
//very useful for regularization and binary stellar evolution
struct binary{
	star s1;
	star s2;

	double dt; //time step of the binary star (inside regularization)
	double ecc; //eccentricity of the binary
	double a; //semimajor axis of the binary
	double period; //orbital period of the binary
	cl_double4 ang_mom; /* binary angular momentum 
	 x y z is direction UNITY VECTOR
	 w is the magnitude angular momentum */
	double binding_energy; //binding energy of the binary system
	

	double newcore; //new core after merger after common envelope evolution.
	double E_tomatch; //binding energy that I have to match after coalescing in common envelope evolution.
	bool coreisCO; //does the core after merging is CO or He? 
};


#endif
