#include <Binarystellarevolution.h>
#include <Stellarevolution.h> //inheritance!!

#include <cmath> //for fabs()
#include <errhand.h>
#include <cstdlib>

using namespace std;

#define errhand_macro(err) errhand(err, __FILE__, __LINE__)

int local_array_bound(float *array, int size, float val){
   int lb = 0; 
	int hb = size-1;
    
   while (lb < hb) {
		int mid = lb + (hb - lb)/2;

      if (array[mid] < val)
         lb = mid + 1;
      else
         hb = mid;
   }
   return lb;
}



int local_search_wr(float *array, int size, float val){
    
    for(int i = 0; i < size-1; i++){
        if(array[i+1] < array[i]){
            if (array[i] > val && array[i+1] < val){
                //ok... first derivative negative and second point found (WR star)
                return i;
            }
        }
    }
    
    return -1;
}




int local_search_wr_co(float *array_co, float *array_he, int size, float val){
    
    for(int i = 0; i < size-1; i++){
        if(array_he[i+1] < array_he[i]){ //first derivatve must be negative for WR stars
            if (array_co[i] < val && array_co[i+1] > val){
                //ok... first derivative negative and second point found (WR star)
                return i;
            }
        }
    }
    
    return -1;
}

int local_search_notwr_co(float *array_co, float *array_he, int size, float val){
    
    for(int i = 0; i < size-1; i++){
        if(array_he[i+1] != 0.0){
            if(array_he[i+1] >= array_he[i]){
                if(array_co[i] < val && array_co[i+1] > val){
                    //ok... first derivative positive and first point (He + H envelope star)
                    return i;
                }
            }
            else return -1; // I am already in the descending part... so I cannot have a match here 
        }
    }
    
    return -1;
}




int local_search_notwr(float *array, int size, float val){
    
    for(int i = 0; i < size-1; i++){
        if(array[i+1] != 0.0){ //He core must be != 0.0
            if(array[i+1] >= array[i]){
                if(array[i] < val && array[i+1] > val){
                    //ok... first derivative positive and first point (He + H envelope star)
                    return i;
                }
            }
            else return -1;
        }
    }
    
    return -1;
}

void Binarystellarevolution::set_new_tloc_he(star *s, star *s_old, double newzams, double m1, double q1){

    int begin, end; //local auxiliary indexes
    
    s->mzams = newzams; //SET THE NEW ZAMS
    cout<<" NEW ZAMS FOUND = "<<newzams<<endl;
   // exit(0);

    get_final_parameters(s);// find the new position in tables

    //restore previous values for the needed parameters
    s->m_t = s_old->m_t;
    s->he_t = s_old->he_t;
    s->co_t = s_old->co_t;
    s->menv = s_old->menv;
    s->maccr = s_old->maccr;
    
    //now for determining the new t_loc we need to require that, considering our way to 
    //interpolate stellar evolution parameters, the new quantities are matched exactly.

    //go to zlow, mup
    begin = global_matrix_istart[s->gmatrixind.s[0]][s->gmatrixind.s[2]-1];
    end = global_matrix_istart[s->gmatrixind.s[0]][s->gmatrixind.s[2]] - 1;
    
    int k1, k2;
    if(s->menv == 0.0 && s->co_t == 0.0) //I want to match a WR star
        k2 = local_search_wr(&stevo_he[begin], end-begin+1, s->he_t) + begin;
    else
        k2 = local_search_notwr(&stevo_he[begin], end-begin+1, s->he_t) + begin;
    
    k1 = k2+1;
    
    //calculate the interpolation coefficients for zlow, mup
    double m2 = (stevo_he[k2] - stevo_he[k1])/(stevo_times[k2] - stevo_times[k1]);
    double q2 = stevo_he[k2] - m2*stevo_times[k2];
    
    double z2 = s->wz.s[0]*s->mzams/newzams;
    double z1 = s->wz.s[1]*s->mzams/newzams;
    //this star does not have CO yet otherwise costart ---> sntime and hestart ----> costart
    
    double t02 = s->neigh_hestart.s[1];
    double t01 = s->neigh_hestart.s[3];
    double tf2 = s->neigh_costart.s[1];
    double tf1 = s->neigh_costart.s[3];
    
    //formula to recover the percentage of life to guarantee that the next interpolations
    //will match exactly what we required (He core in our case)
    double PE = (s->he_t - m1*t01*z1 - m2*t02*z2 - q1*z1 - q2*z2)/(m1*tf1*z1 + m2*tf2*z2 -m1*t01*z1 - m2*t02*z2);
    
    //AND FINALLY... the desired t_loc (and percentage of life) for the next step
    s->t_loc = PE*(s->co_start - s->he_start) + s->he_start;
    s->pl = PE;
    
    stevoevolve(s); //update parameters of this stars at the new local time
    
    return;
    
}


void Binarystellarevolution::set_new_tloc_co(star *s, star *s_old, double newzams, double m1, double q1){

    int begin, end; //local auxiliary indexes
    
    s->mzams = newzams; //SET THE NEW ZAMS

    get_final_parameters(s);// find the new position in tables

    //restore previous values for the needed parameters
    s->m_t = s_old->m_t;
    s->he_t = s_old->he_t;
    s->co_t = s_old->co_t;
    s->menv = s_old->menv;
    s->maccr = s_old->maccr;
    
    //now for determining the new t_loc we need to require that, considering our way to 
    //interpolate stellar evolution parameters, the new quantities are matched exactly.

    //go to zlow, mup
    begin = global_matrix_istart[s->gmatrixind.s[0]][s->gmatrixind.s[2]-1];
    end = global_matrix_istart[s->gmatrixind.s[0]][s->gmatrixind.s[2]] - 1;
    
    int k1, k2;
    k2 = local_array_bound(&stevo_co[begin], end-begin+1, s->co_t) + begin;
    
    k1 = k2+1;
    
    //calculate the interpolation coefficients for zlow, mup
    double m2 = (stevo_co[k2] - stevo_co[k1])/(stevo_times[k2] - stevo_times[k1]);
    double q2 = stevo_co[k2] - m2*stevo_times[k2];
    
    double z2 = s->wz.s[0]*s->mzams/newzams;
    double z1 = s->wz.s[1]*s->mzams/newzams;
    
    //this star does not have CO yet otherwise costart ---> sntime and hestart ----> costart
    double t02 = s->neigh_costart.s[1];
    double t01 = s->neigh_costart.s[3];
    double tf2 = s->lifetimes.s[1];
    double tf1 = s->lifetimes.s[3];
    
    //formula to recover the percentage of life to guarantee that the next interpolations
    //will match exactly what we required (CO core in our case)
    double PE = (s->co_t - m1*t01*z1 - m2*t02*z2 - q1*z1 - q2*z2)/(m1*tf1*z1 + m2*tf2*z2 -m1*t01*z1 - m2*t02*z2);
    
    //AND FINALLY... the desired t_loc (and percentage of life) for the next step
    s->t_loc = PE*(s->sntime - s->co_start) + s->co_start;
    s->pl = PE;
    
    stevoevolve(s); //update parameters of this stars at the new local time
    
    return;
    
}


void Binarystellarevolution::best_match_co(star *s, star *s_old){
    
    cout<<" best match CO "<<endl;
    
    int start_index = s->gmatrixind.s[5]-1; //high Z, Mup
    int begin, end;
    double bestmatch = 1.0e30;
    double newzams, m1, q1;
    
    for(int i = start_index; i < global_matrix_istart[s->gmatrixind.s[3]].size(); i++){
        begin = global_matrix_istart[s->gmatrixind.s[3]][i];
        end = global_matrix_istart[s->gmatrixind.s[3]][i+1] - 1;
        
        int k2, k1;
        if(s->m_t == s->he_t){ // I am looking for a WR star!! no H envelope!! but WITH CO CORE (WR_HG)
            //this means that I need to search a new track which is a WR star as well
            cout<<" search WR co "<<endl;
            k2 = local_search_wr_co(&stevo_co[begin], &stevo_he[begin], end-begin+1, s->co_t) + begin;
            k1 = k2+1;
        }
        else{ //Looking for a CO + He + H envelope track (early AGB)
            k2 = local_search_notwr_co(&stevo_co[begin], &stevo_he[begin], end-begin+1, s->co_t) + begin;
            k1 = k2+1;
        }
        
        
        if(k2 < begin)//I could not find a good match in the current track... go to the next one
            continue;
        else{ //Ok.. .I found a possible match for the current track... inspect it in more detail
            //calculate the exact time when this track has exactly the CO core that I want to match
            double slope = (stevo_co[k2] - stevo_co[k1])/(stevo_times[k2] - stevo_times[k1]);
            double intercept = stevo_co[k2] - slope*stevo_times[k2];
            double newtime = (s->co_t - intercept)/slope; //This is the new time (approximately)
            
            double comatch = slope*newtime + intercept;
            
            double slopem = (stevo_masses[k2] - stevo_masses[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptm = stevo_masses[k2] - slopem*stevo_times[k2];
            double massmatch = slopem*newtime+interceptm;
            
            double slopemhe = (stevo_he[k2] - stevo_he[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptmhe = stevo_he[k2] - slopemhe*stevo_times[k2];
            double hematch = slopemhe*newtime+interceptmhe;
            
            double variationzams; //the smallest value of this variable belong to the best-match track
            
            //if I am a WR star, try to find the track that is closest 
            //in terms of He core
            if(s->m_t == s->he_t){
                
                cout<<"-----------------------------------"<<endl;
                cout<<" my parameters = "<<s->m_t<<"   "<<s->he_t<<"   "<<s->co_t<<endl;
                cout<<" my best match = "<<massmatch<<"   "<<hematch<<"   "<<comatch<<endl;
                
                
                if(slopemhe < 0.0){ //condition for a WR
                    variationzams = fabs(s->he_t - hematch);
                }
                else
                    variationzams = 1.0e30;
                
                cout<<"-----------------------------------"<<endl;
            }
            //if I am not a WR star, try to find the best match with both He and H masses
            else{
                
                cout<<"-----------------------------------"<<endl;
                cout<<" my parameters = "<<s->m_t<<"   "<<s->he_t<<"   "<<s->co_t<<endl;
                cout<<" my best match = "<<massmatch<<"   "<<hematch<<"   "<<comatch<<endl;
                
                if(slopemhe >= 0.0){ //I cannot be a WR star...
                    variationzams = fabs(s->he_t - hematch);// + fabs(s->m_t - massmatch)/massmatch;
                    //match just HE... H in excess goes into the reservoir.
                }
                else
                    variationzams = 1.0e30;
                
                cout<<"-----------------------------------"<<endl;
            }
        
            
            if(variationzams < bestmatch){
                cout<<" GOT IT AS BEST MATCH "<<endl;
                bestmatch = variationzams; 
                newzams = stevo_masses[begin]; 
                m1 = slope; //parameters to exactly recover CO mass
                q1 = intercept;
            }
        }
        
        
    }
    
    //in genere qui calcolo i reservoirs... cioe' quanto scarto rispetto al best match trovato
    //pero' seguo comunque l'evoluzione della stella best match piu vicina....
    //poi i reservoir tornano utili quando c'e' in gioco l'esplosione come supernova
    
    if(bestmatch == 1.0e30){
        cout<<" Could not find the star "<<endl;
        // se ho accresciuto solo H allora l'H accresciuto lo lascio come reservoir
        //H reservoir e' inerte e continuo l'avoluzione della stella considerandone solo
        //gli strati piu' interni... insomma... non cambio traccia...
        //al momento dell'esplosione in SN, pero', tengo conto anche del reservoir
        exit(0);
    }
    
    exit(0);
    set_new_tloc_co(s, s_old, newzams, m1, q1);
    
    
    
    
    
    return;
    
}



void Binarystellarevolution::best_match_he(star *s, star *s_old){
    
    cout<<" BEST MATCH HE "<<endl;
    
    int start_index = s->gmatrixind.s[5]-1; //high Z, Mup
    int begin, end;
    double bestzams = s_old->mzams + s->maccr;
    double bestmatch = 1.0e30;
    double newzams, m1, q1;
    
    cout<<" best zams = "<<bestzams<<endl;
    cout<<s->m_t<<endl;
    cout<<s->he_t<<endl;
    
    for(int i = start_index; i < global_matrix_istart[s->gmatrixind.s[3]].size(); i++){
        begin = global_matrix_istart[s->gmatrixind.s[3]][i];
        end = global_matrix_istart[s->gmatrixind.s[3]][i+1] - 1;
        
        int k2, k1;
        if(s->m_t == s->he_t){ // I am looking for a WR star!! no H envelope!!
            //this means that I need to search a new track which is a WR star as well
            k2 = local_search_wr(&stevo_he[begin], end-begin+1, s->he_t) + begin;
            k1 = k2+1;
        }
        else{ //Looking for a He + H envelope track
            k2 = local_search_notwr(&stevo_he[begin], end-begin+1, s->he_t) + begin;
            k1 = k2+1;
        }
        
        
        if(k2 < begin)//I could not find a good match in the current track... go to the next one
            continue;
        else{ //Ok.. .I found a possible match for the current track... inspect it in more detail
            //calculate the exact time when this track has exactly the HE core that I want to match
            double slope = (stevo_he[k2] - stevo_he[k1])/(stevo_times[k2] - stevo_times[k1]);
            double intercept = stevo_he[k2] - slope*stevo_times[k2];
            double newtime = (s->he_t - intercept)/slope; //This is the new time
            
            double slopem = (stevo_masses[k2] - stevo_masses[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptm = stevo_masses[k2] - slopem*stevo_times[k2];
            double massmatch = slopem*newtime+interceptm;
            
            double variationzams; //the smallest value of this variable belong to the best-match track
            
            //if I iam a WR star, try to find the track that is closest to the
            // OLD zams mass decreased/increased by the removed/accreted mass
            if(s->m_t == s->he_t){
                if(massmatch <= s->he_t)
                    variationzams = fabs(stevo_masses[begin] - bestzams);
                else
                    variationzams = 1.0e30;
            }
            //if I am not a WR star, try to find the best match with the H envelope mass
            else{
                if(massmatch > s->he_t){ //I cannot be a WR star... so massmatch > s->he_t
                    variationzams = fabs((s->m_t - s->he_t) - (massmatch-s->he_t));
                }
                else
                    variationzams = 1.0e30;
            }
        
            
            if(variationzams < bestmatch){
                bestmatch = variationzams; 
                newzams = stevo_masses[begin]; 
                m1 = slope;
                q1 = intercept;
            }
        }
        
        
    }
    
    
    
    set_new_tloc_he(s, s_old, newzams, m1, q1);
    
    
    
    
    
    return;
    
}

void Binarystellarevolution::set_new_tloc_h_only(star *s, star *s_old, int begin, int prev_begin, double M1, double M2){
    
    //try to determine the correct zams mass and the right time
    //and set also the new t_loc
    //this formulas are based on reverse interpolation: given the way to inteprolate,
    //calculate mzams and t_loc to force the next interpolation to match exactly
    //the required mass
    
     double m2 = stevo_masses[begin];
     double m1 = stevo_masses[prev_begin];
     double M = s->m_t;
     
     double A = m1*M1 - m2*M2;
     double B = M*m2*m2 - M*m1*m1 - m1*m2*M1 + m1*m2*M2;
     double C = m1*m1*m2*M - m1*m2*m2*M;
     double newzams = (-B - sqrt(B*B - 4.0*A*C))/(2.0*A);
     
     //prendi gli indici relativi alle nuove m_t considerate come zams
    s->mzams = newzams;
    get_final_parameters(s);

    //restore previous values for the needed parameters
    s->m_t = s_old->m_t;
    s->he_t = s_old->he_t;
    s->co_t = s_old->co_t;
    s->menv = s_old->menv;
    s->maccr = s_old->maccr;
    s->pl = s_old->pl;
    
    s->t_loc = s->pl*s->he_start; //this is still a MS star without he core... so here t_loc is given using the hs_start time
    
    stevoevolve(s); //update parameters of this stars at the new local time
    
    return;
    
}

void Binarystellarevolution::best_match_h_only(star *s, star *s_old){
    
    int start_index = s->gmatrixind.s[1]-1;
    int prevstart, prevend;
    int begin, end;
    double M1, M2;
    
    for(int i = start_index; i < global_matrix_istart[s->gmatrixind.s[0]].size(); i++){
        begin = global_matrix_istart[s->gmatrixind.s[0]][i];
        end = global_matrix_istart[s->gmatrixind.s[0]][i+1] - 1;

        //fix the percentage of life for MS (H only) stars
        //and try to search for the track that match exactly the
        //total mass at that percentage of life
        
        double localtime = s->pl*stevo_times[end];
        int k2 = local_array_bound(&stevo_times[begin], end-begin+1, localtime) + begin;
        int k1 = k2+1;
        
        double m1 = (stevo_masses[k2] - stevo_masses[k1])/(stevo_times[k2] - stevo_times[k1]);
        double q1 = stevo_masses[k2] - m1*stevo_times[k2];
        double trymass = m1*localtime + q1; //mass of the track at the given percentage of life
        
        M2 = trymass;
        if(trymass >= s->m_t) //ok.. I found a range of good ZAMS masses
            break;
        else{
            prevend = end;
            prevstart = begin;
            M1 = trymass;
        }
    }
    
    
    set_new_tloc_h_only(s, s_old, begin, prevstart, M1, M2);
    
    
    
    
    return;
    
}

void Binarystellarevolution::find_new_track(star *s){
    
    cout<<"**************************************"<<endl;
    cout<< "FIND NEW TRACK"<<endl;
    cout<<"**************************************"<<endl;
    
    //save the old ZAMS... we could need it later to remember where the star started from
    star s_old = *s; //variable to save the initial state of the star
    
    //start inspect the tracks starting from the track with initial mzams = m_t
    //so the first new tentative zams will be m_t
    s->mzams = s->m_t;

    //find the indexes in the tables corresponding to the new tentative zams
    get_final_parameters(s); 
    
    //restore previous values for the needed parameters
    s->m_t = s_old.m_t;
    s->he_t = s_old.he_t;
    s->co_t = s_old.co_t;
    s->menv = s_old.menv;
    s->maccr = s_old.maccr;
    s->pl = s_old.pl;
    
    cout<<" TRY TO BEST MATCH "<<endl;
    
    if(s->he_t != 0.0 && s->co_t == 0.0){
        // I have already formed a HE core... so match it EXACTLY for the next track
        //try to match it using the high Z table and Mup
        best_match_he(s, &s_old); //could be WR or He + H envelope star
    }
    else if (s->he_t != 0.0 && s->co_t != 0.0){
        best_match_co(s, &s_old); //could be a WR with CO or an early AGB
    }
    else
        best_match_h_only(s, &s_old);
    

    cout<<"**************************************"<<endl;
    cout<< "FIND NEW TRACK END"<<endl;
    cout<<"**************************************"<<endl;
    
    
    return;
    
}


void Binarystellarevolution::find_new_track_after_CE(binary *bs){
    
    //only one star survives
    
    //save the old ZAMS... we could need it later to remember where the star started from
    star s_old = bs->s1; //variable to save the initial state of the star
    bs->s2.isempty = true; //the second star has been eaten in this case
    //we use star s1 to store all the new parameters
    
    //start inspect the tracks starting from the track with initial mzams = bs->newcore
    //so the first new tentative zams will be bs->newcore
    bs->s1.mzams = bs->newcore;

    //find the indexes in the tables corresponding to the new tentative zams
    get_final_parameters(&bs->s1); 
    
    if(bs->coreisCO){
        bs->s1.co_t = bs->newcore;
        match_CO_and_binding(&bs->s1, bs->E_tomatch);
    }
    else{
        bs->s1.he_t = bs->newcore;
        match_HE_and_binding(&bs->s1, bs->E_tomatch);
    }
        
    return;
    
}

void Binarystellarevolution::match_CO_and_binding(star *s, double energy){
    
    int start_index = s->gmatrixind.s[5]-1; //high Z, Mup
    int begin, end;
    double bestmatch = 1.0e30;
    double newzams, m1, q1;
    
    for(int i = start_index; i < global_matrix_istart[s->gmatrixind.s[3]].size(); i++){
        begin = global_matrix_istart[s->gmatrixind.s[3]][i];
        end = global_matrix_istart[s->gmatrixind.s[3]][i+1] - 1;
        
        int k2, k1;
        //Looking for a CO + He + H envelope track... envelope here is everything but CO
        k2 = local_array_bound(&stevo_co[begin], end-begin+1, s->co_t) + begin;
        k1 = k2+1;
        
        if(k2 < begin)//I could not find a good match in the current track... go to the next one
            continue;
        else{ //Ok.. .I found a possible match for the current track... inspect it in more detail
            //calculate the exact time when this track has exactly the HE core that I want to match
            double slope = (stevo_co[k2] - stevo_co[k1])/(stevo_times[k2] - stevo_times[k1]);
            double intercept = stevo_co[k2] - slope*stevo_times[k2];
            double newtime = (s->co_t - intercept)/slope; //This is the new time
            
            double slopem = (stevo_masses[k2] - stevo_masses[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptm = stevo_masses[k2] - slopem*stevo_times[k2];
            double massmatch = slopem*newtime+interceptm;
            
            double sloper = (stevo_radius[k2] - stevo_radius[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptr = stevo_radius[k2] - sloper*stevo_times[k2];
            double radiusmatch = sloper*newtime+interceptr;

            //calculate the binding energy of the envelope for the current track
            double myenergy =  massmatch*(massmatch-s->co_t)/radiusmatch;
            double variation; //the smallest value of this variable belong to the best-match track

            variation = fabs(myenergy - energy);
            
            if(variation < bestmatch){
                bestmatch = variation; 
                newzams = stevo_masses[begin]; 
                m1 = slope;
                q1 = intercept;
            }
        }
    }
    
    //here s_old must be identical to s... no needed to store the old star
    set_new_tloc_co(s, s, newzams, m1, q1);
    
    
    
    
    
    return;
    
}

void Binarystellarevolution::match_HE_and_binding(star *s, double energy){
    
    int start_index = s->gmatrixind.s[5]-1; //high Z, Mup
    int begin, end;
    double bestmatch = 1.0e30;
    double newzams, m1, q1;
    
    for(int i = start_index; i < global_matrix_istart[s->gmatrixind.s[3]].size(); i++){
        begin = global_matrix_istart[s->gmatrixind.s[3]][i];
        end = global_matrix_istart[s->gmatrixind.s[3]][i+1] - 1;
        
        int k2, k1;
        //Looking for a He + H envelope track
        k2 = local_search_notwr(&stevo_he[begin], end-begin+1, s->he_t) + begin;
        k1 = k2+1;
        
        if(k2 < begin)//I could not find a good match in the current track... go to the next one
            continue;
        else{ //Ok.. .I found a possible match for the current track... inspect it in more detail
            //calculate the exact time when this track has exactly the HE core that I want to match
            double slope = (stevo_he[k2] - stevo_he[k1])/(stevo_times[k2] - stevo_times[k1]);
            double intercept = stevo_he[k2] - slope*stevo_times[k2];
            double newtime = (s->he_t - intercept)/slope; //This is the new time
            
            double slopem = (stevo_masses[k2] - stevo_masses[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptm = stevo_masses[k2] - slopem*stevo_times[k2];
            double massmatch = slopem*newtime+interceptm;
            
            double sloper = (stevo_radius[k2] - stevo_radius[k1])/(stevo_times[k2] - stevo_times[k1]);
            double interceptr = stevo_radius[k2] - sloper*stevo_times[k2];
            double radiusmatch = sloper*newtime+interceptr;

            //calculate the binding energy of the envelope for the current track
            double myenergy =  massmatch*(massmatch-s->he_t)/radiusmatch;
            double variation; //the smallest value of this variable belong to the best-match track

            variation = fabs(myenergy - energy);
            
            if(variation < bestmatch){
                bestmatch = variation; 
                newzams = stevo_masses[begin]; 
                m1 = slope;
                q1 = intercept;
            }
        }
    }
    
    //here s_old must be identical to s... no needed to store the old star
    s->menv = 1.0e30; //be sure that the next routine will not search for a WR star.
    set_new_tloc_he(s, s, newzams, m1, q1);
    
    
    
    
    
    return;
    
}


void Binarystellarevolution::binary_stellar_evolution(binary *bs){


    cout<<" M1 and M2  = "<<bs->s1.m_t<<"   "<<bs->s2.m_t<<endl;
    cout<<" HE1 ad HE2 = "<<bs->s1.he_t<<"   "<<bs->s2.he_t<<endl;
    cout<<" CO1 ad CO2 = "<<bs->s1.co_t<<"   "<<bs->s2.co_t<<endl;
    
    double mold1 = bs->s1.m_t;
    double mold2 = bs->s2.m_t;
    /***************************************************/
    //artificial mass loss for now
    //reservoir of mass to loose = ENVELOPE
    
   /* double deltaM = 0.5*bs->s2.menv;
    cout<<"\n\n delta M = "<<deltaM<<endl;
    if(deltaM == 0.0) return;
    
    //star n2 is the accretor
    bs->s2.menv = bs->s2.menv - deltaM;
    bs->s2.maccr = -deltaM;
    
    //star n1 is the donor
    bs->s1.menv = bs->s1.menv + deltaM;
    bs->s1.maccr = deltaM;
    
    //update total mass after accretion
    bs->s1.m_t = bs->s1.menv + bs->s1.he_t;
	bs->s2.m_t = bs->s2.menv + bs->s2.he_t;*/
    /***************************************************/
    
    bs->s1.m_t = bs->s1.m_t + 10.0;
    bs->s1.he_t = bs->s1.he_t + 10.0;
    //10 solar masses of He.... my star eats a WR_MS star
    
    cout<<" M1 and M2  = "<<bs->s1.m_t<<"   "<<bs->s2.m_t<<endl;
    cout<<" HE1 ad HE2 = "<<bs->s1.he_t<<"   "<<bs->s2.he_t<<endl;
    cout<<" CO1 ad CO2 = "<<bs->s1.co_t<<"   "<<bs->s2.co_t<<endl;
    bs->s1.menv = bs->s1.m_t - bs->s1.he_t;
    bs->s2.menv = bs->s2.m_t - bs->s2.he_t;
    bs->s1.maccr = bs->s1.m_t - mold1;
    bs->s2.maccr = bs->s2.m_t - mold2;
    
    
    
    if(bs->E_tomatch != 0.0){ //in this case only one star survives
        cout<<" LOOKING TO MACH CE "<<endl;
        cout<<"E energy = "<<bs->E_tomatch<<endl;
        exit(0);
        find_new_track_after_CE(bs);
        bs->E_tomatch = bs->newcore = 0.0; //reset parameters
    }
    else{
        find_new_track(&bs->s1);
        find_new_track(&bs->s2);
    }



	return;

}


void Binarystellarevolution::move_through_tracks(binary *bs){
    
    
    return;
    
}


void Binarystellarevolution::set_scores(vector<float> &sc, vector<cl_int2> &sc_ind, int z_ind, int m_ind, star *s){

	   
	sc.erase(sc.begin(), sc.end());
   sc_ind.erase(sc_ind.begin(), sc_ind.end());
   int count = 1;
   int initial_ind = global_matrix_istart[z_ind][m_ind];
   int final_ind   = global_matrix_istart[z_ind][m_ind+count] - 1;

   cout<<initial_ind<<"  final ind = "<<final_ind<<endl;

   for(;;){

      for(int i = initial_ind; i <= final_ind; i++){
 
			double mass_variation = fabs(s->m_t - stevo_masses[i])/s->m_t;
         double he_variation   = fabs(s->he_t - stevo_he[i])/s->he_t;
			if(s->he_t == 0.0)
				he_variation = 0.0;
         double scr = mass_variation + he_variation;


         cl_int2 idxs;
         idxs.s[0] = i;
         idxs.s[1] = m_ind+count - 1; //to recover the initial index of global_matrix_istart

         sc.push_back(scr);
         sc_ind.push_back(idxs);

      }

      count++;
      initial_ind = final_ind + 1;
		int size_global_matrix = static_cast<vector<int>::size_type>(global_matrix_istart[z_ind].size());
      if(m_ind + count == size_global_matrix-1)
         break;

//		if(s->he_t == 0.0 && s->m_t < s->mzams) //do not need to investigate other tracks if helium = 0 and the mass is below the original ZAMS
//			break;



      final_ind = global_matrix_istart[z_ind][m_ind+count] - 1;

//      if(stevo_masses[final_ind] > s->m_t) //final mass is not monothonic with ZAMS.. cannot stop
//         break;
   }



   Quick_Sort(sc, sc_ind, 0, sc.size()-1);

   cout<<" values of scores for zlow and m1 "<<sc[0]<<endl;

   int initmass = global_matrix_istart[z_ind][sc_ind[0].s[1]];
   cout<<" initial NEW track (mass) = "<<stevo_masses[initmass]<<"    time = "<<stevo_times[scores_zlow_ind[0].s[0]]<<endl;
   cout<<" MCO           = "<<stevo_co[sc_ind[0].s[0]]<<endl;
   cout<<" previous MCO  = "<<s->co_t<<endl;
   cout<<" MHE           = "<<stevo_he[sc_ind[0].s[0]]<<endl;
   cout<<" previous MHE  = "<<s->he_t<<endl;
   cout<<" M             = "<<stevo_masses[sc_ind[0].s[0]]<<endl;
   cout<<" previous M    = "<<s->m_t<<endl;

   if(stevo_co[sc_ind[0].s[0]] < s->co_t)
      cout<<" Reservoir CO = "<<s->co_t-stevo_co[sc_ind[0].s[0]]<<endl;


   cout<<"*****************************************"<<endl;


	return;

}





void Binarystellarevolution::set_initial_new_track(double m, double z, cl_int2 *my_index_star, cl_int2 *final_index_star, int *zl, int* zh, int *ml, int *mh){

	//positioning on the track of the star with mass m1_new.
   //zlow and zhigh are fixed... indexes of the masses change!

   //start from a track with mass > m1_new.. do not need to save the lower track
   //need only m2_new_zlow and m2_new_zhigh....I do not need m1 info.



	//start from the first metallicity
	int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
   for(int i = 0; i < metallicity_values_size; i++){
      if(z <= metallicity_values[i]){
         *zh = i;
			*zl = max(0, i-1);
         break;
      }
   }


	int list_masses_zl_size = static_cast<vector<int>::size_type>(list_masses[*zl].size());
    //list masses and tables must be ORDERED in the input files
   for(int i = 0; i < list_masses_zl_size; i++){
      if(m <= list_masses[*zl][i]){
         *ml = i;
         break;
      }
   }

   cout<<" m2_new_zlow = "<<list_masses[*zl][*ml]<<endl;

   if(*ml < 0)
      throw errhand_macro(index_masses_low_not_found);


	int list_masses_zh_size = static_cast<vector<int>::size_type>(list_masses[*zh].size());
   //finding mass indexes for high z   
   for(int i = 0; i < list_masses_zh_size; i++){
      if(m <= list_masses[*zh][i]){
         *mh = i;
         break;
      }
   }

   if(*mh < 0)
      throw errhand_macro(index_masses_high_not_found);

   cout<<" m2_new_zhigh = "<<list_masses[*zh][*mh]<<endl;



   //return the variable that contains the array indexes
   //update the istart variables
   my_index_star->s[0] = global_matrix_istart[*zl][*ml];
   my_index_star->s[1] = global_matrix_istart[*zh][*mh];

   final_index_star->s[0] = global_matrix_istart[*zl][*ml+1] - 1;
   final_index_star->s[1] = global_matrix_istart[*zh][*mh+1] - 1;


	return;

}




void Binarystellarevolution::select_track(star *s, int z_ind, vector<cl_int2> &sc_ind){


   s->co_res = s->co_t-stevo_co[sc_ind[0].s[0]];

      //follow just 1 track on 1 table (table = zlow)
      s->ind.s[0] = sc_ind[0].s[0];
      s->ind.s[1] = sc_ind[0].s[0] + 1;
      s->ind.s[2] = s->ind.s[0];
      s->ind.s[3] = s->ind.s[1];
      //zhigh values do not matter anymore
      s->ind.s[4] = 0;
      s->ind.s[5] = 1;
      s->ind.s[6] = 0;
      s->ind.s[7] = 1;

		s->wm.s[0] = 1.0;
		s->wm.s[1] = 0.0;

      //only zlow matters
		s->wz.s[0] = 1.0;
      s->wz.s[1] = 0.0;

      int final_local_index = global_matrix_istart[z_ind][sc_ind[0].s[1]+1] - 1;
      s->sntime = stevo_times[final_local_index];
      cout<<" NEW ESTIMATED life time = "<<s->sntime<<endl;

      s->m_fin = stevo_masses[final_local_index];
      s->he_fin = stevo_he[final_local_index];

		s->r_fin = stevo_radius[final_local_index];
		s->l_fin = stevo_luminosity[final_local_index];
		s->phase_fin = phase_to_char((int)stevo_phase[final_local_index]);

        //set lifetimes of neighbour tracks
        s->lifetimes.s[0] = stevo_times[final_local_index];
        s->lifetimes.s[1] = stevo_times[final_local_index];
        s->lifetimes.s[2] = stevo_times[final_local_index];
        s->lifetimes.s[3] = stevo_times[final_local_index];
        
      int initmass = global_matrix_istart[z_ind][sc_ind[0].s[1]];

      //CO remains the same if I accreted mass and I am going on a track with a smaller mass
      if(stevo_masses[initmass] > s->mzams){
         s->co_fin = max(s->co_res, stevo_co[final_local_index]);
			s->Dco = 0.0;
		}
		else
			s->Dco = max((s->co_fin - stevo_co[final_local_index]), 0.0f);

		 
		if(stevo_masses[initmass] >= 16.7*s->z + 7.0){ //TRESHOLD for SN explosion... Sandro private communication
			if(to_string(s->sn) == "delayed")
		      delayed_SN_explosion(s);
	
		   if(to_string(s->sn) == "startrack")
		      startrack_SN_explosion(s);

		   if(to_string(s->sn) == "rapid")
		      rapid_SN_explosion(s);

	      set_birth_kick(s);
		}

	   else{ //I have a white dwarf
	      s->m_rem = 0.117*stevo_masses[initmass] + 0.384; // Catàlan et al., 2008, MNRAS 387, 1693
	      s->sntime = max(s->sntime, (float)get_life_time(stevo_masses[initmass])); // this is needed just because PARSEC tables do not contain TP_AGB so 
	      cout<<" Got life time = "<<s->sntime<<endl;
          
          //set lifetimes of neighbour tracks
        s->lifetimes.s[0] = s->sntime;
        s->lifetimes.s[1] = s->sntime;
        s->lifetimes.s[2] = s->sntime; 
        s->lifetimes.s[3] = s->sntime;
          
	      //I do not know the time when I will need to transform stars into white dwarfs. In this way I estimate this time
	      //using the standard STARLAB recipes for lifetimes in different stellar evolution phases.
	   }


      cout<<" NEW ESTIMATED remnant mass = "<<s->m_rem<<endl;
      s->t_rejuv = stevo_times[sc_ind[0].s[0]] - s->t_loc;
		s->t_loc = stevo_times[sc_ind[0].s[0]];



	return;

}




void Binarystellarevolution::find_closest_track(binary *bs){


	cout<<" Trying to find the closest track in the PARSEC tables"<<endl;

	if(bs->s1.he_t == 0.0 && bs->s1.m_t < bs->s1.mzams){ //do not change track!!!
		//if helium mass is null and new mass is less than ZAMS ---> stay on the same track... just move backward in time!!
		set_initial_new_track(bs->s1.mzams, bs->s1.z, &my_index[0], &final_index[0], &zlow[0], &zhigh[0], &m2_new_zlow[0], &m2_new_zhigh[0]);
	}
	if(bs->s2.he_t == 0.0 && bs->s2.m_t < bs->s2.mzams){
		set_initial_new_track(bs->s2.mzams, bs->s2.z, &my_index[1], &final_index[1], &zlow[1], &zhigh[1], &m2_new_zlow[1], &m2_new_zhigh[1]);
	}

	if(bs->s1.he_t != 0.0 || bs->s1.m_t > bs->s1.mzams) //if helium mass != 0 and I accrete Hydrogen ----> change track!!
		set_initial_new_track(bs->s1.m_t, bs->s1.z, &my_index[0], &final_index[0], &zlow[0], &zhigh[0], &m2_new_zlow[0], &m2_new_zhigh[0]);
	if(bs->s2.he_t != 0.0 || bs->s2.m_t > bs->s2.mzams)
		set_initial_new_track(bs->s2.m_t, bs->s2.z, &my_index[1], &final_index[1], &zlow[1], &zhigh[1], &m2_new_zlow[1], &m2_new_zhigh[1]);




	cout<<" new track mass m1, zlow = "<<stevo_masses[my_index[0].s[0]]<<endl;
	cout<<" new track mass m1, zhigh = "<<stevo_masses[my_index[0].s[1]]<<endl;

	cout<<" new track mass m2, zlow = "<<stevo_masses[my_index[1].s[0]]<<endl;
   cout<<" new track mass m2, zhigh = "<<stevo_masses[my_index[1].s[1]]<<endl;


	//select track for star #1 and zlow
	set_scores(scores_zlow, scores_zlow_ind, zlow[0], m2_new_zlow[0], &bs->s1);
	//select track for star #1 and zhigh
	set_scores(scores_zhigh, scores_zhigh_ind, zhigh[0], m2_new_zhigh[0], &bs->s1);

	//update parameters for star #1
	if(fabs(bs->s1.z - metallicity_values[zlow[0]]) < fabs(bs->s1.z - metallicity_values[zhigh[0]]) ){
      cout<<" choosing the star with Z LOW = "<<metallicity_values[zlow[0]]<<endl;
		select_track(&bs->s1, zlow[0], scores_zlow_ind);
	}
	else{
		cout<<" choosing the star with Z HIGH = "<<metallicity_values[zhigh[0]]<<endl;
		select_track(&bs->s1, zhigh[0], scores_zhigh_ind);
	}




	//select track for star #2 and zlow
   set_scores(scores_zlow, scores_zlow_ind, zlow[1], m2_new_zlow[1], &bs->s2);
   //select track for star #2 and zhigh
   set_scores(scores_zhigh, scores_zhigh_ind, zhigh[1], m2_new_zhigh[1], &bs->s2);

   //update parameters for star #2
   if(fabs(bs->s2.z - metallicity_values[zlow[1]]) < fabs(bs->s2.z - metallicity_values[zhigh[1]]) ){
      cout<<" choosing the star with Z LOW = "<<metallicity_values[zlow[1]]<<endl;
      select_track(&bs->s2, zlow[1], scores_zlow_ind);
   }
   else{
      cout<<" choosing the star with Z HIGH = "<<metallicity_values[zhigh[1]]<<endl;
      select_track(&bs->s2, zhigh[1], scores_zhigh_ind);
   }






	return;

}


