#include <Stellarevolution.h>

#include <cstdlib> //for exit(0)
#include <cmath> //for sqrt()

#include <cstring>

using namespace std;

#define errhand_macro(err) errhand(err, __FILE__, __LINE__)


void Stellarevolution::set_array_position(cl_int8 *cpu_stevo_ind, cl_int8 *gmind){


   // cout<<" set array position"<<endl;
    //errhand g(metallicity_too_low, __FILE__, __LINE__);
  //  cout<<" before throw "<<endl;
//	throw errhand_macro(metallicity_too_low);

	//list_masses[index_metallicity][index_mass] -----> value = mass
   //index_metallicity = 0 -----> metallicity = metallicity_values[0]
   //index_mass = 8 -------> mass = list_masses[index_mass][index_metallicity]
   //----------------------> istart = global_matrix_istart[index_mass][index_metallicity]
   //----------------------> first mass and time = stevo_masses[istart] and stevo_times[istart]

	//metallicity_values are orfered from the lowest to the highest
	zlow = -1; zhigh = -1;
	m1_zlow = -1; m2_zlow = -1;
	m1_zhigh = -1; m2_zhigh = -1;

	//cout<<" n. Z available = "<<metallicity_values.size()<<endl;
	int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());

	//check if metallicity is within the allowed range
	if(Z < metallicity_values[0]) 
		throw errhand_macro(metallicity_too_low);
	if(Z > metallicity_values[metallicity_values.size()-1]) 
		throw errhand_macro(metallicity_too_high);

	//start from the first metallicity
	for(int i = 0; i < metallicity_values_size; i++){
		if(Z <= metallicity_values[i]){
			zhigh = i;
			zlow = max(0, i-1);
			break;
		}
	}

	if(zlow < 0 || zhigh < 0) 
		throw errhand_macro(index_metallicity_not_found);


	 //cout<<" zhigh = "<<metallicity_values[zhigh]<<endl;
  // cout<<" zlow  = "<<metallicity_values[zlow]<<endl;
	//cout<<" zhigh zlow = "<<zhigh<<"  "<<zlow<<endl;


	//avoid nan for minimum metallicity (for which zhigh = zlow)
	if(metallicity_values[zhigh] == metallicity_values[0]){
		zhigh = 1;
	}


	  
	//cout<<" metal values = "<<metallicity_values[0]<<"   "<<metallicity_values[1]<<endl;
	//cout<<" zhigh = "<<metallicity_values[zhigh]<<endl;
   //cout<<" zlow  = "<<metallicity_values[zlow]<<endl;
	//cout<<" zhigh zlow = "<<zhigh<<"  "<<zlow<<endl;


	//now that I have determined metallicity values, check mass indexes
	
	int list_masses_zlow_size = static_cast<vector<int>::size_type>(list_masses[zlow].size());
	//list masses and tables must be ORDERED in the input files
	for(int i = 0; i < list_masses_zlow_size; i++){
		if(m <= list_masses[zlow][i]){
			m2_zlow = i;
			m1_zlow = max(0, i-1);
			break;
		}
	}

	//cout<<" m1_zlow = "<<list_masses[zlow][m1_zlow]<<endl;
	//cout<<" m2_zlow = "<<list_masses[zlow][m2_zlow]<<endl;

	if(m1_zlow < 0 || m2_zlow < 0)    
      throw errhand_macro(index_masses_low_not_found);


	int list_masses_zhigh_size = static_cast<vector<int>::size_type>(list_masses[zhigh].size());
	//finding mass indexes for high z	
	for(int i = 0; i < list_masses_zhigh_size; i++){
      if(m <= list_masses[zhigh][i]){
         m2_zhigh = i;
         m1_zhigh = max(0, i-1);
         break;
      }
   }
   
   if(m1_zhigh < 0 || m2_zhigh < 0) 
      throw errhand_macro(index_masses_high_not_found);

	//cout<<" m1_zhigh = "<<list_masses[zhigh][m1_zhigh]<<endl;
   //cout<<" m2_zhigh = "<<list_masses[zhigh][m2_zhigh]<<endl;


   //global_matrix_istart[zlow][m1_zlow] - 1 = last element to consider for zlow/m1_zlow
   //for the first element to consider I should go to the previous star but if
   //m1_zlow corresponds to 0.1 going back to the previous star means that I have to
   //go back to the previous metallicity and I have to consider the last mass of that metallicity.
   

	//return the variable that contains the array indexes
	//set the istart variables
   
   if(m1_zlow-1 >= 0)
        cpu_stevo_ind->s[0] = global_matrix_istart[zlow][m1_zlow-1];
   else{
       if(zlow-1 < 0)
            cpu_stevo_ind->s[0] = 0; //well, I am on the very first point
        else{
            int prev_index = list_masses[zlow-1].size()-1;
            cpu_stevo_ind->s[0] = global_matrix_istart[zlow-1][prev_index];
        }
   }
   
       
   // cout<<"stevo masses low = "<<stevo_masses[cpu_stevo_ind->s[0]]<<endl;
    
	cpu_stevo_ind->s[1] = global_matrix_istart[zlow][m1_zlow] - 1;
   // cout<<"stevo masses low = "<<stevo_masses[cpu_stevo_ind->s[1]]<<endl;
   // cout<<" initial final m1 zlow = "<<cpu_stevo_ind->s[0]<<"   "<<cpu_stevo_ind->s[1]<<endl;
    
    
    
    //this should never happen.... but, just in case........
    if(m2_zlow-1 >= 0)
        cpu_stevo_ind->s[2] = global_matrix_istart[zlow][m2_zlow-1];
   else{
       if(zlow-1 < 0)
            cpu_stevo_ind->s[2] = 0; //well, I am on the very first point
        else{
            int prev_index = list_masses[zlow-1].size()-1;
            cpu_stevo_ind->s[2] = global_matrix_istart[zlow-1][prev_index];
        }
   }
    
   cpu_stevo_ind->s[3] = global_matrix_istart[zlow][m2_zlow] - 1;
  // cout<<" initial final m2 zlow = "<<cpu_stevo_ind->s[2]<<"   "<<cpu_stevo_ind->s[3]<<endl;

   
   
   
   
   if(m1_zhigh-1 >= 0)
        cpu_stevo_ind->s[4] = global_matrix_istart[zhigh][m1_zhigh-1];
   else{
        if(zhigh-1 < 0) //this should never happen
            cpu_stevo_ind->s[4] = 0;
        else{
            int prev_index = list_masses[zhigh-1].size()-1;
            cpu_stevo_ind->s[4] = global_matrix_istart[zhigh-1][prev_index];
        }
   }
   
    cpu_stevo_ind->s[5] = global_matrix_istart[zhigh][m1_zhigh] - 1;
 //  cout<<" initial final m1 zhigh = "<<cpu_stevo_ind->s[4]<<"   "<<cpu_stevo_ind->s[5]<<endl;
   
   
   
   
   if(m2_zhigh-1 >= 0)
        cpu_stevo_ind->s[6] = global_matrix_istart[zhigh][m2_zhigh-1];
   else{
        if(zhigh-1 < 0)
            cpu_stevo_ind->s[6] = 0;
        else{
            int prev_index = list_masses[zhigh-1].size()-1;
            cpu_stevo_ind->s[6] = global_matrix_istart[zhigh-1][prev_index];
        }
   }
   
   
   cpu_stevo_ind->s[7] = global_matrix_istart[zhigh][m2_zhigh] - 1;
  // cout<<" initial final m2 zhigh = "<<cpu_stevo_ind->s[6]<<"   "<<cpu_stevo_ind->s[7]<<endl;
   
	//element m1_zlow+1 should always exist.... because global_matrix_istart is updated
	//before calling the if(in.eof()) break;
	final_ind.s[0] = global_matrix_istart[zlow][m1_zlow] - 1;//next istart minus 1
	final_ind.s[1] = global_matrix_istart[zlow][m2_zlow] - 1;//next istart minus 1
	final_ind.s[2] = global_matrix_istart[zhigh][m1_zhigh] - 1;
	final_ind.s[3] = global_matrix_istart[zhigh][m2_zhigh] - 1;
    
   gmind->s[0] = zlow;
   gmind->s[1] = m1_zlow;
   gmind->s[2] = m2_zlow;
   gmind->s[3] = zhigh;
   gmind->s[4] = m1_zhigh;
   gmind->s[5] = m2_zhigh;
    

	return;

}


void Stellarevolution::set_weights(cl_float2 *cpu_mass_weights, cl_float2 *cpu_metal_weights){


	//get metal and mass weights
   //mass weights
	//using zlow or zhigh here is the same thing
	float mup = list_masses[zhigh][m2_zhigh];
	float mdown = list_masses[zhigh][m1_zhigh];

    
    //stevo mass weights: the mass of a star will be evaluated following:
    // m = A*m1 + B*m2  where m2 > m1, (A,B) = (1,0) for m = m1,
    // (A,B) = (0,1) for m = m2. and A + B = 1 always.
    // If we start from A = m/m1 and B = m/m2 we obtain, adjusting
    // A = (m2-m)/(m2-m1) and B = (m-m1)/(m2-m1). Thus, we have LINEAR weights in m.
    
    //RATIONAL weights work definitely better interpolating stellar evolution tracks.
    //We still need to figure out why, but we tested it.
    
    //Let's start from A = m1/m. For m = m1 we have A = 1 (this is what we want).
    //Still, for m = m2 we have A = m1/m2 but we want A = 0 in this case.
    //Well we can try A = m1/m - m1/m2. For m = m2 we have A = 0 (nice) but now
    //for m = m1 we have A = (m2-m1)/m2 (but we want A = 1 in this case).
    //Let's try A = (m1/m - m1/m2)*m2/(m2-m1). We have A = 1 for m = m1 and
    //A = 0 for m = m2. Perfect. We can do the same thing for B.
    //So we have A = (m2*m1/m - m1)/(m2-m1) and B = (m2 - m2*m1/m)/(m2-m1).
    
    
   local_mass_weights.s[0] = cpu_mass_weights->s[0] = fabs(mdown - mdown*mup/m)/(mup-mdown);
   local_mass_weights.s[1] = cpu_mass_weights->s[1] = fabs(mup   - mdown*mup/m)/(mup-mdown);
   //metal weights
	float zup = metallicity_values[zhigh];
	float zdown = metallicity_values[zlow];
	//cout<<" zup = "<<zup<<endl;
	//cout<<" zlow = "<<zdown<<endl;
    
    //correction = (m/(cpu_mass_weights->s[0]*mdown + cpu_mass_weights->s[1]*mup)); we must multiply the mass result for this correction because
    //with the new weights we need to impose that cpu_mass_weights->s[0]*mdown + cpu_mass_weights->s[1]*mup = m... which is not true by default.
    
    //Weights for metallicity follow the LINEAR trend (it may change in the future)
   local_metal_weights.s[0] = cpu_metal_weights->s[0] = (zup - Z) / ( (Z - zdown) + (zup - Z));
   local_metal_weights.s[1] = cpu_metal_weights->s[1] = (Z - zdown)  / ( (Z - zdown) + (zup - Z));

/*
   cout<<" m1 weight = "<<cpu_mass_weights->s[0]<<endl;
   cout<<" m2 weight = "<<cpu_mass_weights->s[1]<<endl;

   cout<<" z1 weight = "<<cpu_metal_weights->s[0]<<endl;
   cout<<" z2 weight = "<<cpu_metal_weights->s[1]<<endl;
*/

	return;

}

void Stellarevolution::delayed_SN_explosion(star *s){

	float final_CO = s->co_fin;
	float final_mass = s->m_fin;

	if(final_CO < 2.5){
		alpha = 0.0;  // dependence on MCO of f_fb
      beta = 0.2/(final_mass-1.2);  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.2; // fixed proto mass
   }

   else if(final_CO >= 2.5 && final_CO < 3.5){
		alpha = 0.0;  // dependence on MCO of f_fb
      beta = (0.5*final_CO - 1.05)/(final_mass-1.2);  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.2; // fixed proto mass
   }

   else if(final_CO >= 3.5 && final_CO < 6.0){
		alpha = 0.133 - (0.093/(final_mass - 1.3));  // dependence on MCO of f_fb
      beta = -11.0*alpha + 1.0;  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.3; // fixed proto mass
   }

   else if(final_CO >= 6.0 && final_CO < 11.0){
		alpha = 0.133 - (0.093/(final_mass - 1.4));  // dependence on MCO of f_fb
      beta = -11.0*alpha + 1.0;  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.4; // fixed proto mass
   }

   else{
		alpha = 0.0;  // dependence on MCO of f_fb
      beta = 1.0;  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.6; // fixed proto mass
   }

   
	float mrembar = s->pisn_correction*(gamma*final_CO + delta + (alpha*final_CO + beta)*(final_mass-gamma*final_CO - delta));


   if(mrembar <= 3.0) //we have a NS
		s->m_rem = 6.6667*(sqrt(1.0+0.3*mrembar)-1.0);  // to GPU with correct mass unit !!
   else // transformation to BH
		s->m_rem = 0.9*mrembar;

	//fallback material... needed to assign birth kicks to compact remnants
	s->fallback = alpha * final_CO + beta;


	return;

}


void Stellarevolution::startrack_SN_explosion(star *s){


	float final_CO = s->co_fin;
   float final_mass = s->m_fin;


	if(final_CO < 4.82){
		gamma = 0.0;
      alpha = 0.0;
      beta = 0.0;
      delta = 1.5;
   }

   else if(final_CO >= 4.82 && final_CO < 5.0){
		gamma = 0.0;
      alpha = 0.0;
      beta = 0.0;
      delta = 2.11;
   }

   else if(final_CO >= 5.0 && final_CO < 6.31){
		gamma = 0.0;
      alpha = 0.378;
      beta = -1.889;
      delta = 2.11;
   }

   else if(final_CO >= 6.31 && final_CO < 6.75){
		gamma = 0.69;
      alpha = 0.378;
      beta = -1.889;
      delta = -2.26;
   }

   else if(final_CO >= 6.75 && final_CO < 7.6){
		gamma = 0.37;
      alpha = 0.378;
      beta = -1.889;
      delta = -0.07;
   }

   else{
		gamma = 0.37;
      alpha = 0.0;
      beta = 1.0;
      delta = -0.07;
   }

      
	float mrembar = s->pisn_correction*(gamma*final_CO + delta + (alpha*final_CO + beta)*(final_mass-gamma*final_CO - delta));

      if(mrembar <= 3.0) //we have, indeed, a NS
			s->m_rem = 6.6667*(sqrt(1.0+0.3*mrembar)-1.0); // to GPU with correct mass unit !!
      else // transformation to BH
         s->m_rem = 0.9*mrembar;

		//fallback material... needed to assign birth kicks to compact remnants
   s->fallback = alpha * final_CO + beta;


	return;

}


void Stellarevolution::rapid_SN_explosion(star *s){

	float final_CO = s->co_fin;
   float final_mass = s->m_fin;


	if(final_CO < 2.5){
		alpha = 0.0f;  // dependence on MCO of f_fb
		beta = 0.2/(final_mass-1.0);  // fixed f_fb
		gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.0; // fixed proto mass
	}

	else if(final_CO >= 2.5 && final_CO < 6.0){
		alpha = 0.0;  // dependence on MCO of f_fb
      beta = (0.286*final_CO - 0.514)/(final_mass - 1.0);  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.0; // fixed proto mass
   }

   else if(final_CO >= 6.0 && final_CO < 7.0){
      alpha = 0.0;  // dependence on MCO of f_fb
      beta = 1.0;  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.0; // fixed proto mass
   }

   else if(final_CO >= 7.0 && final_CO < 11.0){
      alpha = 0.25 - (1.275/(final_mass-1.0));  // dependence on MCO of f_fb
      beta = -11.0*alpha + 1.0;  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.0; // fixed proto mass
   }

   else{
      alpha = 0.0;  // dependence on MCO of f_fb
      beta = 1.0;  // fixed f_fb
      gamma = 0.0; //dependence on MCO of proto mass
      delta = 1.0; // fixed proto mass
   }

        
	float mrembar = s->pisn_correction*(gamma*final_CO + delta + (alpha*final_CO + beta)*(final_mass-gamma*final_CO - delta));

		if(mrembar <= 3.0) //we have, indeed, a NS
			s->m_rem = 6.6667*(sqrt(1.0+0.3*mrembar)-1.0);  // to GPU with correct mass unit !!
      else // transformation to BH
			s->m_rem = 0.9*mrembar;

		//fallback material... needed to assign birth kicks to compact remnants
   s->fallback = alpha * final_CO + beta;



	return;

}


void Stellarevolution::pair_instability(star *s){

	//default: no correction, i.e. pisn_correction factor = 1.0;
	s->pisn_correction = 1.0;

	//Helium fraction
	float he_frac = s->he_fin/s->m_fin;
	//k_param (pisn_correction dependence upon helium fraction)
	float k_param = 0.67*he_frac + 0.1;
	
	//pulsation pair-instability supernova: pulses
	if(s->he_fin > 32.0 && s->he_fin < 64.0){

		//use Table 2 fit from Woosley 2006
		if(he_frac < 0.9){
	
			if(s->he_fin <= 37.0)
				s->pisn_correction = (k_param - 1.0)/5.0*s->he_fin + (37.0 - 32.0*k_param)/5.0;
			else if (s->he_fin > 37.0 && s->he_fin <= 60.0)
				s->pisn_correction = k_param;
			else
				s->pisn_correction = -(k_param/4.0)*s->he_fin + 16.0*k_param;
		}
	
		//use WR table 1 fit from Woosley 2016
		else{

			if(s->he_fin <= 37.0)
				s->pisn_correction = (0.5226*he_frac-0.52974)*(s->he_fin-32.0) + 1.0;
			else if (s->he_fin > 37.0 && s->he_fin <= 56.0){
				float val = (0.5226*he_frac-0.52974)*5.0 + 1.0;
				if(val < 0.82916)
					s->pisn_correction = val;
				else
					s->pisn_correction = (-0.1381*he_frac+0.1309)*(s->he_fin-56.0) + 0.82916;
			}
			else
				s->pisn_correction = -0.103645*s->he_fin + 6.63328;
			/*
			if (s->he_fin > 32.0 && s->he_fin <= 56.0)
				s->pisn_correction = -0.00714*s->he_fin + 1.229;
			else
				s->pisn_correction = -0.103645*s->he_fin + 6.63328;
			*/
		}

	}


	//pair-instability supernova: disintegrated
	else if (s->he_fin >= 64.0 && s->he_fin < 135.0)
		s->pisn_correction = 0.0;
	//end of PISN... again standard direct collapse
	else
		s->pisn_correction = 1.0;

//	s->pisn_correction = 1.0;
//

//	if(s->pisn_correction != 1.0 && s->pisn_correction != 0.0)
//		myoutf<<s->z<<"   "<<s->mzams<<endl;
//	if(s->pisn_correction == 0.0)
//		fil<<s->z<<"   "<<s->mzams<<endl;

	return;

}


void Stellarevolution::supernova(star *s){


    float corr_zlow = s->mzams/(s->wm.s[0]*stevo_masses[s->ind.s[0]] + s->wm.s[1]*stevo_masses[s->ind.s[2]]);
    float corr_zup = s->mzams/(s->wm.s[0]*stevo_masses[s->ind.s[4]] + s->wm.s[1]*stevo_masses[s->ind.s[6]]);

	float final_CO_1 = local_mass_weights.s[0]*stevo_co[final_ind.s[0]] + local_mass_weights.s[1]*stevo_co[final_ind.s[1]];
	float final_CO_2 = local_mass_weights.s[0]*stevo_co[final_ind.s[2]] + local_mass_weights.s[1]*stevo_co[final_ind.s[3]];

	s->co_fin = local_metal_weights.s[0]*(corr_zlow*final_CO_1) + local_metal_weights.s[1]*(corr_zup*final_CO_2);
    
	float final_mass_1 = local_mass_weights.s[0]*(stevo_masses[final_ind.s[0]]) + local_mass_weights.s[1]*(stevo_masses[final_ind.s[1]]);
	float final_mass_2 = local_mass_weights.s[0]*(stevo_masses[final_ind.s[2]]) + local_mass_weights.s[1]*(stevo_masses[final_ind.s[3]]);

	s->m_fin = local_metal_weights.s[0]*(corr_zlow*final_mass_1) + local_metal_weights.s[1]*(corr_zup*final_mass_2);

	float final_He_1 = local_mass_weights.s[0]*stevo_he[final_ind.s[0]] + local_mass_weights.s[1]*stevo_he[final_ind.s[1]];
   float final_He_2 = local_mass_weights.s[0]*stevo_he[final_ind.s[2]] + local_mass_weights.s[1]*stevo_he[final_ind.s[3]];

   s->he_fin = local_metal_weights.s[0]*(corr_zlow*final_He_1) + local_metal_weights.s[1]*(corr_zup*final_He_2);

	float final_rad_1 = local_mass_weights.s[0]*stevo_radius[final_ind.s[0]] + local_mass_weights.s[1]*stevo_radius[final_ind.s[1]];
   float final_rad_2 = local_mass_weights.s[0]*stevo_radius[final_ind.s[2]] + local_mass_weights.s[1]*stevo_radius[final_ind.s[3]];

   s->r_fin = local_metal_weights.s[0]*(corr_zlow*final_rad_1) + local_metal_weights.s[1]*(corr_zup*final_rad_2);

	float final_lum_1 = local_mass_weights.s[0]*stevo_luminosity[final_ind.s[0]] + local_mass_weights.s[1]*stevo_luminosity[final_ind.s[1]];
   float final_lum_2 = local_mass_weights.s[0]*stevo_luminosity[final_ind.s[2]] + local_mass_weights.s[1]*stevo_luminosity[final_ind.s[3]];

   s->l_fin = local_metal_weights.s[0]*(corr_zlow*final_lum_1) + local_metal_weights.s[1]*(corr_zup*final_lum_2);


/*


	if(local_mass_weights.s[0] >= local_mass_weights.s[1]){
      if(local_metal_weights.s[0] >= local_metal_weights.s[1]){
         //using track at zlow and m1
         s->phase_fin = phase_to_char((int)stevo_phase[final_ind.s[0]]);
      }  
   }
   
   if(local_mass_weights.s[0] < local_mass_weights.s[1]){
      if(local_metal_weights.s[0] >= local_metal_weights.s[1]){
         //using track at zlow and m2
         s->phase_fin = phase_to_char((int)stevo_phase[final_ind.s[1]]);
      }
   }
   
   if(local_mass_weights.s[0] < local_mass_weights.s[1]){
      if(local_metal_weights.s[0] < local_metal_weights.s[1]){
         //using track at zhigh and m2
         s->phase_fin = phase_to_char((int)stevo_phase[final_ind.s[3]]);
      }
   }

   if(local_mass_weights.s[0] >= local_mass_weights.s[1]){
      if(local_metal_weights.s[0] < local_metal_weights.s[1]){
         //using track at zhigh and m1
         s->phase_fin = phase_to_char((int)stevo_phase[final_ind.s[2]]);
      }
   }
   
   */


	//check for PISN or PPISN occurrence
	pair_instability(s);

/*
	cout<<" Definitive final CO core    = "<<s->co_fin<<endl;
	cout<<" Definitive final mass       = "<<s->m_fin<<endl;
	cout<<" Definitive final radius     = "<<s->r_fin<<endl;
	cout<<" Definitive final luminosity = "<<s->l_fin<<endl;*/

	if(SN_model == "delayed")
		delayed_SN_explosion(s);

	if(SN_model == "startrack")
		startrack_SN_explosion(s);

	if(SN_model == "rapid")
		rapid_SN_explosion(s);

	//cout<<" Definitive compact remnant = "<<s->m_rem<<endl;
	//cout<<" FALLBACK material = "<<s->fallback<<endl;


	return;

}



void Stellarevolution::set_remnant_time(float *SN_time){

	double final_time_1 = local_mass_weights.s[0]*stevo_times[final_ind.s[0]] + local_mass_weights.s[1]*stevo_times[final_ind.s[1]];
	double final_time_2 = local_mass_weights.s[0]*stevo_times[final_ind.s[2]] + local_mass_weights.s[1]*stevo_times[final_ind.s[3]];

	*SN_time = local_metal_weights.s[0]*final_time_1 + local_metal_weights.s[1]*final_time_2;

	//cout<<" Estimated life time = "<<*SN_time<<endl;

	return;

}


float Stellarevolution::maxwellian_cdf(float x, float sigma){
	float a = x/sigma;
	return (erf(0.7071067811865475*a) - 0.7978845608028654*a*exp(-0.5*a*a));
}

float Stellarevolution::maxwellian_pdf(float x, float sigma){
	float a = x/sigma;
	return (0.7978845608028654*a*a/sigma*exp(-0.5*a*a));
}


void Stellarevolution::set_birth_kick(star *s){

	//maxwellian velocities with sigma = 265 km/s
	//initialize random number generator
	srand(time(NULL));
	float birth_kick = 0.0;

   float yrand = rand()/(float)RAND_MAX;
   float sigma = 265.0; //observed in km/s... Hobbs et al., 2005, MNRAS 360 974
   float moda = sqrt(2.0)*sigma; // to inizialize the Newton-Raphson method

   //doing sampling of the cumulative distribution function (cdf)... which ranges from 0 to 1.
   //From a random value of y, I get the corresponding x value.. so I need to solve the equation
   // cdf(x) - random_value = 0... I can use Newton Raphson
   // x1 = x0 - (cdf(x0) - random_value)/Pdf(x0) where Pdf(x) = d/dx[cdf(x)]

   float x1 = moda; //initial test value
   float x0 = moda;

   //Newton Raphson iterations 
   for(int i = 0; i < 10; i++){
      x1 = x0 - (maxwellian_cdf(x0, sigma) - yrand)/maxwellian_pdf(x0, sigma);
      if(fabs(x1 - x0)/fabs(x1) < 1.0e-6)
         break;
      x0 = x1;
   }


	if(fabs(x1 - x0)/fabs(x1) > 1.0e-6)
		throw errhand_macro(newton_method_error);


   birth_kick  = (1.0-s->fallback)*x1; //correct kick for the amount of fallback
   //cout<<" BIRTH KICK (km/s) : "<<birth_kick<<endl;

   //generate random orientation for the velocity kick

   float random = rand()/(float)RAND_MAX;
   s->vkick.s[2] = 2.0 * birth_kick * random - birth_kick;  // to GPU with correct velocity unit !!
   //cout<<" birth kick Z = "<<s->vkick.s[2]<<endl;
   float vel2d = sqrt(birth_kick*birth_kick - s->vkick.s[2]*s->vkick.s[2]);
   random = rand()/(float)RAND_MAX;
   float teta = 2.0 * M_PI * random;

   s->vkick.s[0] = vel2d * cos(teta);  // to GPU with correct velocity unit !!
   s->vkick.s[1] = vel2d * sin(teta);  // to GPU with correct velocity unit !!

   //cout<<"birth kick X e Y : "<<s->vkick.s[0]<<"   "<<s->vkick.s[1]<<endl;

   return;


}



//input must be given in solar masses
//Z is the absolute metallicity.. not in solar units
void Stellarevolution::get_final_parameters(star *s){

	SN_model = s->sn;
	m = s->mzams;
	Z = s->z;
	s->countHG = 0;
	s->mHE_dot_set = false;
	s->RLF = false;
	s->isempty = false;

	//cout<<" read mass = "<<m<<endl;
	//cout<<" read metallicity = "<<Z<<endl;

	//cout<<" stevo masses size = "<<stevo_masses.size()<<endl;
	//get the correct indexes of this star
	set_array_position(&s->ind, &s->gmatrixind);
	//get the correct weights of this star
	set_weights(&s->wm, &s->wz);

    /*
	cout<<" mass 1, t=0, z low  = "<<stevo_masses[s->ind.s[0]]<<endl;
	cout<<" mass 1, t=1, z low  = "<<stevo_masses[s->ind.s[1]]<<endl;

	cout<<" mass 2, t=0, z low  = "<<stevo_masses[s->ind.s[2]]<<endl;
   cout<<" mass 2, t=1, z low  = "<<stevo_masses[s->ind.s[3]]<<endl;

	cout<<" mass 1, t=0, z high = "<<stevo_masses[s->ind.s[4]]<<endl;
   cout<<" mass 1, t=1, z high = "<<stevo_masses[s->ind.s[5]]<<endl;

   cout<<" mass 2, t=0, z high = "<<stevo_masses[s->ind.s[6]]<<endl;
   cout<<" mass 2, t=1, z high = "<<stevo_masses[s->ind.s[7]]<<endl;

	cout<<" final mass 1 zlow   = "<<stevo_masses[final_ind.s[0]]<<endl;
	cout<<" final mass 2 zlow   = "<<stevo_masses[final_ind.s[1]]<<endl;

	cout<<" final mass 1 zhigh  = "<<stevo_masses[final_ind.s[2]]<<endl;
	cout<<" final mass 2 zhigh  = "<<stevo_masses[final_ind.s[3]]<<endl;

	cout<<" final CO 1 zlow = "<<stevo_co[final_ind.s[0]]<<endl;
	cout<<" final CO 2 zlow = "<<stevo_co[final_ind.s[1]]<<endl;

	cout<<" final CO 1 zhigh = "<<stevo_co[final_ind.s[2]]<<endl;
	cout<<" final CO 2 zhigh = "<<stevo_co[final_ind.s[3]]<<endl;

	cout<<" final time 1 zlow = "<<stevo_times[final_ind.s[0]]<<endl;
   cout<<" final time 2 zlow = "<<stevo_times[final_ind.s[1]]<<endl;

   cout<<" final time 1 zhigh = "<<stevo_times[final_ind.s[2]]<<endl;
   cout<<" final time 2 zhigh = "<<stevo_times[final_ind.s[3]]<<endl;
*/
	//initialize other initial parameters
	
	float lum_zlow = local_mass_weights.s[0]*stevo_luminosity[s->ind.s[0]] + local_mass_weights.s[1]*stevo_luminosity[s->ind.s[2]];
	float lum_zhigh = local_mass_weights.s[0]*stevo_luminosity[s->ind.s[4]] + local_mass_weights.s[1]*stevo_luminosity[s->ind.s[6]];
	s->l_t = local_metal_weights.s[0]*lum_zlow + local_metal_weights.s[1]*lum_zhigh;

	float rad_zlow = local_mass_weights.s[0]*stevo_radius[s->ind.s[0]] + local_mass_weights.s[1]*stevo_radius[s->ind.s[2]];
   float rad_zhigh = local_mass_weights.s[0]*stevo_radius[s->ind.s[4]] + local_mass_weights.s[1]*stevo_radius[s->ind.s[6]];
   s->r_t = local_metal_weights.s[0]*rad_zlow + local_metal_weights.s[1]*rad_zhigh;

	s->he_t = s->co_t = 0.0;

	s->m_t = s->mzams;

    s->phase = (char*)"Not a Star";
    
    int bsephase = get_phase_BSE(s);
    s->phase = phase_to_char(bsephase);
    /*
	if(local_mass_weights.s[0] >= local_mass_weights.s[1]){
		if(local_metal_weights.s[0] >= local_metal_weights.s[1]){
			//using track at zlow and m1
			s->phase = phase_to_char((int)stevo_phase[s->ind.s[0]]);
		}
	}

	if(local_mass_weights.s[0] <= local_mass_weights.s[1]){
      if(local_metal_weights.s[0] >= local_metal_weights.s[1]){
         //using track at zlow and m2
         s->phase = phase_to_char((int)stevo_phase[s->ind.s[2]]);
      }  
   }

	if(local_mass_weights.s[0] <= local_mass_weights.s[1]){
      if(local_metal_weights.s[0] <= local_metal_weights.s[1]){
         //using track at zhigh and m2
         s->phase = phase_to_char((int)stevo_phase[s->ind.s[6]]);
      }
   }

	if(local_mass_weights.s[0] >= local_mass_weights.s[1]){
      if(local_metal_weights.s[0] <= local_metal_weights.s[1]){
         //using track at zhigh and m1
         s->phase = phase_to_char((int)stevo_phase[s->ind.s[4]]);
      }
   }
   */


	//cout<<" Initialization = "<<s->phase<<endl;



	//get SN explosion time (if any)
	
	//solar masses here
	if(m < 0.1) throw errhand_macro(star_too_light);
	// Here, I could insert an upper limit for the mass of a star...


	s->sntime = 0.0;
	set_remnant_time(&s->sntime);
    
    
    //set lifetimes of neighbour tracks
    s->lifetimes.s[0] = stevo_times[final_ind.s[0]];
    s->lifetimes.s[1] = stevo_times[final_ind.s[1]];
    s->lifetimes.s[2] = stevo_times[final_ind.s[2]];
    s->lifetimes.s[3] = stevo_times[final_ind.s[3]];

	if(m >= 16.7*Z + 7){ //TRESHOLD for SN explosion... Sandro private communication
		supernova(s); //see Spera et al. 2015, MNRAS  
		set_birth_kick(s);
	}
	else{ //I have a white dwarf
		s->m_rem = 0.117*s->mzams + 0.384; // Catàlan et al., 2008, MNRAS 387, 1693
		s->sntime = max(s->sntime, (float)get_life_time()); // this is needed just because PARSEC tables do not contain TP_AGB so 
        
        //set new lifetimes for neighbour tracks... Sandro does not have them because he misses the TP-AGB phases
        //(stars that are not going to explode as SN)
        s->lifetimes.s[0] = max(s->lifetimes.s[0], (float)get_life_time(stevo_masses[s->ind.s[0]]));
        s->lifetimes.s[1] = max(s->lifetimes.s[1], (float)get_life_time(stevo_masses[s->ind.s[2]]));
        s->lifetimes.s[2] = max(s->lifetimes.s[2], (float)get_life_time(stevo_masses[s->ind.s[4]]));
        s->lifetimes.s[3] = max(s->lifetimes.s[3], (float)get_life_time(stevo_masses[s->ind.s[6]]));
        
		//cout<<" Got life time = "<<s->sntime<<endl;
		//I do not know the time when I will need to transform stars into white dwarfs. In this way I estimate this time
		//using the standard STARLAB recipes for lifetimes in different stellar evolution phases.
        m = s->mzams; //recover zams...
	}
	
	set_coandhe_times(s);

	//cout<<" M remnant is = "<<s->m_rem<<endl;
	//cout<<" I will die at time = "<<s->sntime<<endl;

	return;

}

void Stellarevolution::set_coandhe_times(star *mystar){
    
    //with saveind = i-1 I am starting He and CO from the 0 values.
    
    int saveind = 0;
    mystar->neigh_hestart.s[0] = mystar->neigh_costart.s[0] = -1.0;
    //inspect first track: zlow, mlow... check when mhe becomes != 0
    for(int i = mystar->ind.s[0]; i <= mystar->ind.s[1]; i++){
        if(stevo_he[i] != 0.0){
            mystar->neigh_hestart.s[0] = stevo_times[i-1];
            saveind = i-1; //restart from here when inspecting CO
            break;
        }
    }
    for(int i = saveind; i <= mystar->ind.s[1]; i++){
        if(stevo_co[i] != 0.0){
            mystar->neigh_costart.s[0] = stevo_times[i-1];
            break;
        }
    }
    
    if(mystar->neigh_hestart.s[0] == -1.0 || mystar->neigh_costart.s[0] == -1.0)
        throw errhand_macro("Cannot find He or CO start for the selected track: zlow, mlow. Not in tables!");
    
    
    
    
    
    saveind = 0;
    mystar->neigh_hestart.s[1] = mystar->neigh_costart.s[1] = -1.0;
    //inspect first track: zlow, mhigh... check when mhe becomes != 0
    for(int i = mystar->ind.s[2]; i <= mystar->ind.s[3]; i++){
        if(stevo_he[i] != 0.0){
            mystar->neigh_hestart.s[1] = stevo_times[i-1];
            saveind = i-1; //restart from here when inspecting CO
            break;
        }
    }
    for(int i = saveind; i <= mystar->ind.s[3]; i++){
        if(stevo_co[i] != 0.0){
            mystar->neigh_costart.s[1] = stevo_times[i-1];
            break;
        }
    }
    
    if(mystar->neigh_hestart.s[1] == -1.0 || mystar->neigh_costart.s[1] == -1.0)
        throw errhand_macro("Cannot find He or CO start for the selected track: zlow, mhigh. Not in tables!");
    
    
    
    
    saveind = 0;
    mystar->neigh_hestart.s[2] = mystar->neigh_costart.s[2] = -1.0;
    //inspect first track: zhigh, mlow... check when mhe becomes != 0
    for(int i = mystar->ind.s[4]; i <= mystar->ind.s[5]; i++){
        if(stevo_he[i] != 0.0){
            mystar->neigh_hestart.s[2] = stevo_times[i-1];
            saveind = i-1; //restart from here when inspecting CO
            break;
        }
    }
    for(int i = saveind; i <= mystar->ind.s[5]; i++){
        if(stevo_co[i] != 0.0){
            mystar->neigh_costart.s[2] = stevo_times[i-1];
            break;
        }
    }
    
    if(mystar->neigh_hestart.s[2] == -1.0 || mystar->neigh_costart.s[2] == -1.0)
        throw errhand_macro("Cannot find He or CO start for the selected track: zhigh, mlow. Not in tables!");



    
    
    saveind = 0;
    mystar->neigh_hestart.s[3] = mystar->neigh_costart.s[3] = -1.0;
    //inspect first track: zhigh, mhigh... check when mhe becomes != 0
    for(int i = mystar->ind.s[6]; i <= mystar->ind.s[7]; i++){
        if(stevo_he[i] != 0.0){
            mystar->neigh_hestart.s[3] = stevo_times[i-1];
            saveind = i-1; //restart from here when inspecting CO
            break;
        }
    }
    for(int i = saveind; i <= mystar->ind.s[7]; i++){
        if(stevo_co[i] != 0.0){
            mystar->neigh_costart.s[3] = stevo_times[i-1];
            break;
        }
    }
    
    if(mystar->neigh_hestart.s[3] == -1.0 || mystar->neigh_costart.s[3] == -1.0)
        throw errhand_macro("Cannot find He or CO start for the selected track: zhigh, mhigh. Not in tables!");
    
    
    
    
    //now determine my hestart and costart
    float corr_zlow = mystar->mzams/(mystar->wm.s[0]*stevo_masses[mystar->ind.s[0]] + mystar->wm.s[1]*stevo_masses[mystar->ind.s[2]]);
    float corr_zup = mystar->mzams/(mystar->wm.s[0]*stevo_masses[mystar->ind.s[4]] + mystar->wm.s[1]*stevo_masses[mystar->ind.s[6]]);

	float costart_zlow  = local_mass_weights.s[0]*mystar->neigh_costart.s[0] + local_mass_weights.s[1]*mystar->neigh_costart.s[1];
	float costart_zhigh = local_mass_weights.s[0]*mystar->neigh_costart.s[2] + local_mass_weights.s[1]*mystar->neigh_costart.s[3];

	mystar->co_start = local_metal_weights.s[0]*(corr_zlow*costart_zlow) + local_metal_weights.s[1]*(corr_zup*costart_zhigh);
    
    
    float hestart_zlow  = local_mass_weights.s[0]*mystar->neigh_hestart.s[0] + local_mass_weights.s[1]*mystar->neigh_hestart.s[1];
	float hestart_zhigh = local_mass_weights.s[0]*mystar->neigh_hestart.s[2] + local_mass_weights.s[1]*mystar->neigh_hestart.s[3];

	mystar->he_start = local_metal_weights.s[0]*(corr_zlow*hestart_zlow) + local_metal_weights.s[1]*(corr_zup*hestart_zhigh);
    
    
    return;
    
}


double Stellarevolution::get_base_agb_lum(){

	double lum_giant = get_giant_lum();
	double lum_agb = 2454.7;

	if (m > 2.3)
		lum_agb = 6.5*pow(min(m, 20.0f), 3.25);

	double base_agb_lum = max(lum_agb, lum_giant);
	return base_agb_lum;

}

double Stellarevolution::get_ms_time(){

	double ms_time = (2550.0 + 667.0*pow(m,2.5) + pow(m,4.5)) / (0.0327*pow(m,1.5) + 0.346*pow(m,4.5));
	return ms_time;

}

double Stellarevolution::get_hg_time(){

	double ms_time = get_ms_time();
	double hg_time = 0.54*ms_time/(m*(m - 2.1) + 23.3);
	return hg_time;
}


double Stellarevolution::get_base_gb_lum(){

	double k, l;
	double log_m = log10(m);

	if (m < 1.334){
		k = 0.2594 + 0.1348*log_m;
		l = 0.144 - 0.8333*log_m;
	}
	else{
		k = 0.092088 + 0.059338*log_m;
		l = -0.05713 + log_m*(0.3756 -0.1744);
	}

	double lum_base_ms = get_base_ms_lum();

	return lum_base_ms*pow(10.0, k + l);

}

double Stellarevolution::get_base_ms_lum(){

	double lum_base_ms;
	if (m <= 1.093)
      lum_base_ms = (1.107*pow(m, 3.0) + 240.7*pow(m, 9.0)) / (1.0 + 281.9*pow(m, 4.0));
   else
      lum_base_ms = (13990.0*pow(m, 5.0)) /(pow(m, 4.0) + 2151.0*pow(m, 2.0) + 3908.0*m + 9536.0);

	return lum_base_ms;

}

double Stellarevolution::get_giant_lum(){

	double giant_lum = (2.15 + 0.22*pow(m, 3.0))*pow(m, 2.0) / (5.0e-6*pow(m, 4.0)+1.4e-2*m*m + 1.0);
	return giant_lum;

}


double Stellarevolution::get_gb_time(){

	double ms_time = get_ms_time();
	double giant_lum = get_giant_lum();
	double base_agb_lum = get_base_agb_lum();

	double gb_time = 0.15*ms_time - 0.15*ms_time*pow((giant_lum/base_agb_lum), 0.855);
	return gb_time;

}


double Stellarevolution::get_he_giant_lum(){

	double lum_agb_base = get_base_agb_lum();
	double lum_he_giant = 49.0*pow(m, 0.364) + 0.86*pow(m, 4.0);

	double he_giant_lum = min(lum_agb_base, lum_he_giant);
	return he_giant_lum;

}

double Stellarevolution::get_he_time(){

	double base_ms_lum = get_base_ms_lum();
	double lum_he_giant = get_he_giant_lum();
	double ms_time = get_ms_time();

	double he_time = ms_time*base_ms_lum/(lum_he_giant*(pow(m, 0.42) + 0.8));
	return he_time;

}


double Stellarevolution::get_agb_time(){

	double ms_time = get_ms_time();
	double gb_time = get_gb_time();
	double b_time = 0.15*ms_time*pow(get_giant_lum()/(m*(4.0e3 + 5.0e2*m)), 0.855);
	double agb_time = 0.15*ms_time - gb_time - b_time;
	return agb_time;
}


double Stellarevolution::get_life_time(){

	double lifetime = 0.0;
	lifetime += main_sequence_time = get_ms_time();
	lifetime += hertzsprung_gap_time = get_hg_time();
	lifetime += giant_branch_time = get_gb_time();
	lifetime += helium_burning_time = get_he_time();
	lifetime += agb_time = get_agb_time();

	return lifetime;

}

double Stellarevolution::get_life_time(double newzams){

	double lifetime;
	m = newzams;
	lifetime = get_life_time();

	return lifetime;
}




void Stellarevolution::load_available_metallicities(){

	//load a file that contains the list of 
	//metallicities and corresponding files
	
	//check if in is open.. if so, close it!
	if(in.is_open())
		in.close();

	in.open(Z_file.c_str(), ios::in);
	if(!in) throw errhand_macro(no_metallicity_file);

	//cout<<" load metallicity values from file "<<Z_file.c_str()<<endl;

	for(;;){
		elements_per_line = 0; // count (and check) elements in each line
    
		getline(in, line);
      if(in.eof()) break;
      stream.str(line);


      if(stream >> value){ //read first path.. where to find its metallicity files
			metallicity_paths.push_back(value);
         stream.clear();
         elements_per_line++;
      }

		if(stream >> value){ //read first metallicity
			metallicity_values.push_back(string_to_number<float>(value, &err));
			//cout<<" found metallicity = "<<value<<endl;
			if(err)  throw errhand_macro(wrong_conversion);
         stream.clear();
         elements_per_line++;
		}

		if(elements_per_line != 2){
         cout<<" Elements is "<<elements_per_line<<" which is different from 2.. calling exception"<<endl;
         cout<<" Issue reading file line "<<line_number<<" of file containing metallicities and corresponfing files"<<endl;
         throw errhand_macro(wrong_elements_line);
      }

	line_number++;

	}

	in.close();

	//order arrays of metallicities and corresponding files
	//from the lowest to the highest
	Quick_Sort(metallicity_values, metallicity_paths, 0, metallicity_values.size()-1); //from c++ class utilities

	return;

}


void Stellarevolution::read_mass_list(int index){

	//first of all read the list of masses for this metallicity

	vector<float> row_masses;
	int count = 0;

	line = metallicity_paths[index] + "/listmass.dat";
	//cout<<" Opening file = "<<line<<endl;
   in.open(line.c_str(), ios::in);
   if(!in) throw errhand_macro(no_listmass_file);

   for(;;){

		getline(in, line);
      if(in.eof()) break;
		stream.clear(); //clear the error state of stringstream (if any)
      stream.str(line);

      for(;;){ //keep reading the line till its last element
			if(stream >> value){ //read first path.. where to find its metallicity files
				row_masses.push_back(string_to_number<float>(value, &err));
            stream.clear(); //clear the error state of stringstream (if any)
				count++;
         }
         else
				break;
      }

   }

	//list_masses[index_metallicity][index_mass] -----> value = mass
	//index_metallicity = 0 -----> metallicity = metallicity_values[0]
	//index_mass = 8 -------> mass = list_masses[index_mass][index_metallicity]
	//----------------------> istart = global_matrix_istart[index_mass][index_metallicity]
	//----------------------> first mass and time = stevo_masses[istart] and stevo_times[istart]
	

	//cout<<"row size = "<<row_masses.size()<<endl;

	list_masses.push_back(row_masses);
	global_matrix_istart.push_back(vector<int>(row_masses.size())); //add an empty row vector of size row_masses.size().
	//This implies that I am ready to collect data for the metallicity correponding to "index".
	//Each row corresponds to different metallicities.
	
	in.close();

	return;

}

void Stellarevolution::load_stevo_arrays(){


	load_available_metallicities();

	//read line by line all the martices
	//from the lowest metallicity to the highest
	
	//check if in is open.. if so, close it!
	if(in.is_open())
      in.close();


	//global_matrix_istart[index_mass][index_metallicity] ---> value = istart;
	//start loading all the metallicity files: need mass, time, CO
	

	load_mass_files();
	load_time_files();
	load_CO_files();
	load_He_files();
	load_radius_files();
	load_luminosity_files();
	load_phase_files();


return;

}


void Stellarevolution::load_mass_files(){



	//read mass file;
	total_count_per_metallicity = 0;

	int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
   for(int i = 0; i < metallicity_values_size; i++){

      //read masses and allocate space in the arrays
      read_mass_list(i);

//		cout<<" sono qui dentro"<<endl;

      line_number = 0;

      //read mass matrix;
      line = metallicity_paths[i] + "/MassMatrix.dat";
      //cout<<" Mass matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_massmatrix_file);

      for(;;){
         elements_per_line = 0; // count (and check) elements in each line

//			cout<<" prima "<<endl;
         //global_matrix_istart[index_metallicity][index_mass] ---> value = istart;
//         global_matrix_istart[i][line_number] = total_count_per_metallicity;

//       cout<<global_matrix_istart[i][line_number]<<endl;
//			cout<<"dopo"<<endl;


         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_masses.push_back(string_to_number<float>(value, &err));
               stream.clear();
               elements_per_line++;
               total_count_per_metallicity++;
            }
            else
               break;
         }


         global_matrix_istart[i][line_number] = total_count_per_metallicity;

         line_number++;
//			cout<<" line = "<<line_number<<endl;

      }

      in.close(); //read the mass_matrix file for the following metallicity

   }



	return;

}


void Stellarevolution::load_time_files(){


	int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
	//read time files
   for(int i = 0; i < metallicity_values_size; i++){

      //read Time matrix;
      line = metallicity_paths[i] + "/TimeMatrix.dat";
      //cout<<" Time matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_timematrix_file);

      for(;;){

         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_times.push_back(string_to_number<float>(value, &err));
               stream.clear();
            }
            else
               break;
         }

      }

      in.close(); //read the mass_matrix file for the following metallicity

   }


	return;

}



void Stellarevolution::load_CO_files(){


	int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
	//read CO files
   for(int i = 0; i < metallicity_values_size; i++){


      //read CO matrix;
      line = metallicity_paths[i] + "/COMatrix.dat";
      //cout<<" CO matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_COmatrix_file);

      for(;;){

         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_co.push_back(string_to_number<float>(value, &err));
               stream.clear();
            }
            else
               break;
         }

      }

      in.close(); //read the mass_matrix file for the following metallicity

   }



	return;

}



void Stellarevolution::load_He_files(){


	int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
   //read He files
   for(int i = 0; i < metallicity_values_size; i++){


      //read He matrix;
      line = metallicity_paths[i] + "/HEMatrix.dat";
      //cout<<" HE matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_HEmatrix_file);

      for(;;){

         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_he.push_back(string_to_number<float>(value, &err));
               stream.clear();
            }
            else
               break;
         }

      }

      in.close(); //read the mass_matrix file for the following metallicity

   }



   return;

}




void Stellarevolution::load_radius_files(){


   int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
   //read Radius files
   for(int i = 0; i < metallicity_values_size; i++){


      //read He matrix;
      line = metallicity_paths[i] + "/RadiusMatrix.dat";
      //cout<<" Radius matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_Radiusmatrix_file);

      for(;;){

         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_radius.push_back(string_to_number<float>(value, &err));
               stream.clear();
            }
            else
               break;
         }

      }

      in.close(); //read the radius_matrix file for the following metallicity

   }



   return;

}

void Stellarevolution::load_phase_files(){


   int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
   //read Radius files
   for(int i = 0; i < metallicity_values_size; i++){


      //read He matrix;
      line = metallicity_paths[i] + "/TypeMatrix.dat";
      //cout<<" Type matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_Typematrix_file);

      for(;;){

         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_phase.push_back(string_to_number<int>(value, &err));
               stream.clear();
            }
            else
               break;
         }

      }

      in.close(); //read the radius_matrix file for the following metallicity

   }



   return;

}





void Stellarevolution::load_luminosity_files(){


   int metallicity_values_size = static_cast<vector<int>::size_type>(metallicity_values.size());
   //read Luminosity files
   for(int i = 0; i < metallicity_values_size; i++){


      //read He matrix;
      line = metallicity_paths[i] + "/LuminosityMatrix.dat";
      //cout<<" Luminosity matrix file = "<<line<<endl;
      //open the first input file
      in.open(line.c_str(), ios::in);
      if(!in) throw errhand_macro(no_Luminositymatrix_file);

      for(;;){

         getline(in, line);
         if(in.eof()) break;
         stream.clear();
         stream.str(line);


         for(;;){ //keep reading the line till its last element
            if(stream >> value){ //read first path.. where to find its metallicity files
               stevo_luminosity.push_back(string_to_number<float>(value, &err));
               stream.clear();
            }
            else
               break;
         }

      }

      in.close(); //read the luminosity_matrix file for the following metallicity

   }



   return;

}


void Stellarevolution::stevoevolve(star *s){
    
    
    //system_time is the new time, my_local_time is the time of the star, before reaching the new time (system_time)
    
    //set new t_loc
    s->t_loc = s->t_loc + s->time_step;
    
    double my_perc_m;
    float prev_co = s->co_t;
    float prev_he = s->mHE_prev = s->he_t;
    float prev_m = s->m_t;
    
    if(s->mHE_dot != 0.0 && !s->mHE_dot_set){
        s->mHE_dot_init = s->mHE_dot;
        s->mHE_dot_set = true;
    }
    
    if(s->t_loc <= s->he_start)
        s->pl = my_perc_m = s->t_loc/s->he_start;
    else if (s->t_loc <= s->co_start)
        s->pl = my_perc_m = (s->t_loc-s->he_start)/(s->co_start - s->he_start);
    else
        s->pl = my_perc_m = (s->t_loc-s->co_start)/(s->sntime - s->co_start);
    
    
    if(s->t_loc >= s->sntime){
        s->m_t = s->m_rem;
        s->menv = 0.0; //set envelope to 0.0 in the case of a compact remnant
        int myphaseBSE = get_phase_BSE(s);
        s->phase = phase_to_char(myphaseBSE);
        

        s->he_t = 0.0;
        s->co_t = 0.0;
        s->r_t = 0.0; //in this place we should take into account of the luminosity of the 
        //luminous and radius of compact remnants... in the meanwhile they are equal to 0.
        s->l_t = 0.0;
        return;
    }
    
    
    //corrections for the newly used weights for the interpolation
    float corr_zlow = s->mzams/(s->wm.s[0]*stevo_masses[s->ind.s[0]] + s->wm.s[1]*stevo_masses[s->ind.s[2]]);
    float corr_zup = s->mzams/(s->wm.s[0]*stevo_masses[s->ind.s[4]] + s->wm.s[1]*stevo_masses[s->ind.s[6]]);
    
    if(s->t_loc <= s->he_start){
        
        s->he_t = s->co_t = 0.0; //still not need to update He and CO
        s->mHE_dot = s->mCO_dot = 0.0;
        
        float time1, time2, time3, time4;
        int k1, k2, k3, k4, k5, k6, k7, k8;
        float ttarget1, ttarget2, ttarget3, ttarget4;
        
        get_Ks(my_perc_m, 0.0, s->neigh_hestart.s[0], s->ind.s[0], s->ind.s[1], &k1, &k2, &ttarget1);
        get_Ks(my_perc_m, 0.0, s->neigh_hestart.s[1], s->ind.s[2], s->ind.s[3], &k3, &k4, &ttarget2);
        get_Ks(my_perc_m, 0.0, s->neigh_hestart.s[2], s->ind.s[4], s->ind.s[5], &k5, &k6, &ttarget3);
        get_Ks(my_perc_m, 0.0, s->neigh_hestart.s[3], s->ind.s[6], s->ind.s[7], &k7, &k8, &ttarget4);   
        
        time1 = 1.0/(stevo_times[k2] - stevo_times[k1]);
        time2 = 1.0/(stevo_times[k4] - stevo_times[k3]);
        time3 = 1.0/(stevo_times[k6] - stevo_times[k5]);
        time4 = 1.0/(stevo_times[k8] - stevo_times[k7]);
        
        //GET NEW MASS///////////////////////////////////////////////////////////////////////
        s->m_t = get_new_value(time1, stevo_masses[k2]-stevo_masses[k1], stevo_times[k1], stevo_masses[k1], ttarget1,
                               time2, stevo_masses[k4]-stevo_masses[k3], stevo_times[k3], stevo_masses[k3], ttarget2, 
                               time3, stevo_masses[k6]-stevo_masses[k5], stevo_times[k5], stevo_masses[k5], ttarget3,
                               time4, stevo_masses[k8]-stevo_masses[k7], stevo_times[k7], stevo_masses[k7], ttarget4,
                               s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        s->m_dot = (s->m_t - prev_m)*s->time_step;
        
        //GET NEW RADIUS/////////////////////////////////////////////////////////////////////
        s->r_t  = get_new_value(time1, stevo_radius[k2]-stevo_radius[k1], stevo_times[k1], stevo_radius[k1], ttarget1,
                                time2, stevo_radius[k4]-stevo_radius[k3], stevo_times[k3], stevo_radius[k3], ttarget2, 
                                time3, stevo_radius[k6]-stevo_radius[k5], stevo_times[k5], stevo_radius[k5], ttarget3,
                                time4, stevo_radius[k8]-stevo_radius[k7], stevo_times[k7], stevo_radius[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        //GET NEW LUMINOSITY/////////////////////////////////////////////////////////////////////
        s->l_t  = get_new_value(time1, stevo_luminosity[k2]-stevo_luminosity[k1], stevo_times[k1], stevo_luminosity[k1], ttarget1,
                                time2, stevo_luminosity[k4]-stevo_luminosity[k3], stevo_times[k3], stevo_luminosity[k3], ttarget2, 
                                time3, stevo_luminosity[k6]-stevo_luminosity[k5], stevo_times[k5], stevo_luminosity[k5], ttarget3,
                                time4, stevo_luminosity[k8]-stevo_luminosity[k7], stevo_times[k7], stevo_luminosity[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        
        //GET THE NEW PHASE.... a little bit tricky for now... waiting for the fractional phase from Sandro's tracks
        //If I have fractional phases then I can use the get_new_value function
        //get_new_phase(s, k1, k2, k3, k4, k5, k6, k7, k8, ttarget1, ttarget2, ttarget3, ttarget4);
        
    }
    
    else if(s->t_loc <= s->co_start){
        
        s->co_t = 0.0; //still not need to update CO, but I have to update Helium
        s->mCO_dot = 0.0;
        float time1, time2, time3, time4;
        int k1, k2, k3, k4, k5, k6, k7, k8;
        float ttarget1, ttarget2, ttarget3, ttarget4;
        
        get_Ks(my_perc_m, s->neigh_hestart.s[0], s->neigh_costart.s[0], s->ind.s[0], s->ind.s[1], &k1, &k2, &ttarget1);
        get_Ks(my_perc_m, s->neigh_hestart.s[1], s->neigh_costart.s[1], s->ind.s[2], s->ind.s[3], &k3, &k4, &ttarget2);
        get_Ks(my_perc_m, s->neigh_hestart.s[2], s->neigh_costart.s[2], s->ind.s[4], s->ind.s[5], &k5, &k6, &ttarget3);
        get_Ks(my_perc_m, s->neigh_hestart.s[3], s->neigh_costart.s[3], s->ind.s[6], s->ind.s[7], &k7, &k8, &ttarget4);
        
        
        time1 = 1.0/(stevo_times[k2] - stevo_times[k1]);
        time2 = 1.0/(stevo_times[k4] - stevo_times[k3]);
        time3 = 1.0/(stevo_times[k6] - stevo_times[k5]);
        time4 = 1.0/(stevo_times[k8] - stevo_times[k7]);
        
        //GET NEW MASS///////////////////////////////////////////////////////////////////////
        s->m_t = get_new_value(time1, stevo_masses[k2]-stevo_masses[k1], stevo_times[k1], stevo_masses[k1], ttarget1,
                               time2, stevo_masses[k4]-stevo_masses[k3], stevo_times[k3], stevo_masses[k3], ttarget2, 
                               time3, stevo_masses[k6]-stevo_masses[k5], stevo_times[k5], stevo_masses[k5], ttarget3,
                               time4, stevo_masses[k8]-stevo_masses[k7], stevo_times[k7], stevo_masses[k7], ttarget4,
                               s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        s->m_dot = (s->m_t - prev_m)*s->time_step;
        
        //GET NEW RADIUS/////////////////////////////////////////////////////////////////////
        s->r_t  = get_new_value(time1, stevo_radius[k2]-stevo_radius[k1], stevo_times[k1], stevo_radius[k1], ttarget1,
                                time2, stevo_radius[k4]-stevo_radius[k3], stevo_times[k3], stevo_radius[k3], ttarget2, 
                                time3, stevo_radius[k6]-stevo_radius[k5], stevo_times[k5], stevo_radius[k5], ttarget3,
                                time4, stevo_radius[k8]-stevo_radius[k7], stevo_times[k7], stevo_radius[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        //GET NEW LUMINOSITY/////////////////////////////////////////////////////////////////////
        s->l_t  = get_new_value(time1, stevo_luminosity[k2]-stevo_luminosity[k1], stevo_times[k1], stevo_luminosity[k1], ttarget1,
                                time2, stevo_luminosity[k4]-stevo_luminosity[k3], stevo_times[k3], stevo_luminosity[k3], ttarget2, 
                                time3, stevo_luminosity[k6]-stevo_luminosity[k5], stevo_times[k5], stevo_luminosity[k5], ttarget3,
                                time4, stevo_luminosity[k8]-stevo_luminosity[k7], stevo_times[k7], stevo_luminosity[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        //GET NEW HELIUM/////////////////////////////////////////////////////////////////////
        s->he_t  = get_new_value(time1, stevo_he[k2]-stevo_he[k1], stevo_times[k1], stevo_he[k1], ttarget1,
                                time2, stevo_he[k4]-stevo_he[k3], stevo_times[k3], stevo_he[k3], ttarget2, 
                                time3, stevo_he[k6]-stevo_he[k5], stevo_times[k5], stevo_he[k5], ttarget3,
                                time4, stevo_he[k8]-stevo_he[k7], stevo_times[k7], stevo_he[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        s->mHE_dot = (s->he_t - prev_he)*s->time_step;
        
        
        //GET THE NEW PHASE.... a little bit tricky for now... waiting for the fractional phase from Sandro's tracks
        //If I have fractional phases then I can use the get_new_value function
       // get_new_phase(s, k1, k2, k3, k4, k5, k6, k7, k8, ttarget1, ttarget2, ttarget3, ttarget4);
        
    }
    
    else{
        
        float time1, time2, time3, time4;
        int k1, k2, k3, k4, k5, k6, k7, k8;
        float ttarget1, ttarget2, ttarget3, ttarget4;
        
        get_Ks(my_perc_m, s->neigh_costart.s[0], s->lifetimes.s[0], s->ind.s[0], s->ind.s[1], &k1, &k2, &ttarget1);
        get_Ks(my_perc_m, s->neigh_costart.s[1], s->lifetimes.s[1], s->ind.s[2], s->ind.s[3], &k3, &k4, &ttarget2);
        get_Ks(my_perc_m, s->neigh_costart.s[2], s->lifetimes.s[2], s->ind.s[4], s->ind.s[5], &k5, &k6, &ttarget3);
        get_Ks(my_perc_m, s->neigh_costart.s[3], s->lifetimes.s[3], s->ind.s[6], s->ind.s[7], &k7, &k8, &ttarget4);
        
        time1 = 1.0/(stevo_times[k2] - stevo_times[k1]);
        time2 = 1.0/(stevo_times[k4] - stevo_times[k3]);
        time3 = 1.0/(stevo_times[k6] - stevo_times[k5]);
        time4 = 1.0/(stevo_times[k8] - stevo_times[k7]);
        
        //GET NEW MASS///////////////////////////////////////////////////////////////////////
        s->m_t = get_new_value(time1, stevo_masses[k2]-stevo_masses[k1], stevo_times[k1], stevo_masses[k1], ttarget1,
                               time2, stevo_masses[k4]-stevo_masses[k3], stevo_times[k3], stevo_masses[k3], ttarget2, 
                               time3, stevo_masses[k6]-stevo_masses[k5], stevo_times[k5], stevo_masses[k5], ttarget3,
                               time4, stevo_masses[k8]-stevo_masses[k7], stevo_times[k7], stevo_masses[k7], ttarget4,
                               s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        s->m_dot = (s->m_t - prev_m)*s->time_step;
        
        //GET NEW RADIUS/////////////////////////////////////////////////////////////////////
        s->r_t  = get_new_value(time1, stevo_radius[k2]-stevo_radius[k1], stevo_times[k1], stevo_radius[k1], ttarget1,
                                time2, stevo_radius[k4]-stevo_radius[k3], stevo_times[k3], stevo_radius[k3], ttarget2, 
                                time3, stevo_radius[k6]-stevo_radius[k5], stevo_times[k5], stevo_radius[k5], ttarget3,
                                time4, stevo_radius[k8]-stevo_radius[k7], stevo_times[k7], stevo_radius[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        //GET NEW LUMINOSITY/////////////////////////////////////////////////////////////////////
        s->l_t  = get_new_value(time1, stevo_luminosity[k2]-stevo_luminosity[k1], stevo_times[k1], stevo_luminosity[k1], ttarget1,
                                time2, stevo_luminosity[k4]-stevo_luminosity[k3], stevo_times[k3], stevo_luminosity[k3], ttarget2, 
                                time3, stevo_luminosity[k6]-stevo_luminosity[k5], stevo_times[k5], stevo_luminosity[k5], ttarget3,
                                time4, stevo_luminosity[k8]-stevo_luminosity[k7], stevo_times[k7], stevo_luminosity[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        //GET NEW HELIUM/////////////////////////////////////////////////////////////////////
        s->he_t  = get_new_value(time1, stevo_he[k2]-stevo_he[k1], stevo_times[k1], stevo_he[k1], ttarget1,
                                time2, stevo_he[k4]-stevo_he[k3], stevo_times[k3], stevo_he[k3], ttarget2, 
                                time3, stevo_he[k6]-stevo_he[k5], stevo_times[k5], stevo_he[k5], ttarget3,
                                time4, stevo_he[k8]-stevo_he[k7], stevo_times[k7], stevo_he[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        s->mHE_dot = (s->he_t - prev_he)*s->time_step;
        
        //GET NEW CARBON-OXYGEN/////////////////////////////////////////////////////////////////////
        s->co_t  = get_new_value(time1, stevo_co[k2]-stevo_co[k1], stevo_times[k1], stevo_co[k1], ttarget1,
                                time2, stevo_co[k4]-stevo_co[k3], stevo_times[k3], stevo_co[k3], ttarget2, 
                                time3, stevo_co[k6]-stevo_co[k5], stevo_times[k5], stevo_co[k5], ttarget3,
                                time4, stevo_co[k8]-stevo_co[k7], stevo_times[k7], stevo_co[k7], ttarget4,
                                s->wm.s[0], s->wm.s[1], s->wz.s[0]*corr_zlow, s->wz.s[1]*corr_zup);
        
        s->mCO_dot = (s->co_t - prev_co)*s->time_step;
        
        
        //GET THE NEW PHASE.... a little bit tricky for now... waiting for the fractional phase from Sandro's tracks
        //If I have fractional phases then I can use the get_new_value function
        //get_new_phase(s, k1, k2, k3, k4, k5, k6, k7, k8, ttarget1, ttarget2, ttarget3, ttarget4);
        
    }
    
    if(s->he_t < prev_he){
        s->he_t = prev_he;
        s->mHE_dot = 0.0;
    }
    
    if(s->co_t < prev_co){
        s->co_t = prev_co;
        s->mCO_dot = 0.0;
    }
    
    if(s->m_t <= s->he_t){ //WR star... lost H envelope
        s->he_t = s->m_t;
        s->mHE_dot = s->m_dot;
    }
    
    
    
    
    int mytype_BSE = get_phase_BSE(s);
    s->phase = phase_to_char(mytype_BSE);
    
    
    
    
    //Calculate the derived properties once evolution is complete
    //Calculating the stars temperature
    s->T_t = pow(3.828e26 * s->l_t / (4.0 * M_PI * pow(s->r_t * 6.957e8, 2) * 5.67e-8), 0.25);
    
    //Calculating the stars sbsolute magnitude
    s->abs_Mag_t = 4.74 - 2.5 * log10(s->l_t);
    
    //update envelope mass
    s->menv = s->m_t - s->he_t;
    s->mcore = s->he_t;
    
    if(s->he_t == s->m_t){ //WR star
        s->menv = s->he_t - s->co_t; //He envelope
        s->mcore = s->co_t;
    }
    
    if(s->mHE_dot > s->mHE_dot_init)
        s->mHE_dot_init = s->mHE_dot; //set it to the maximum slope
    
    
    return;
    
}


int Stellarevolution::get_phase_BSE(star *s){
    
    //We do not have HE WD (type 10), Naked helium giant (type 9),
    //TP-AGB (type 6), First giant branch (type 3)
    
    int current_type = phase_to_int((char*)s->phase);
    
    if(s->t_loc >= s->sntime){         
        if(s->m_rem > 3.0)
            return 14; //black hole
        else if (s->m_rem < 3.0 && s->mzams > 8.0)
            return 13; //neutron star
        else if (s->m_rem <= 1.4 && s->mzams < 6.0)
            return 11; // CO white dwarf
        else if (s->m_rem <= 1.4 && s->mzams >= 6.0)
            return 12; // O/Ne white dwarf
    }
    
    if(s->he_t == 0.0 && s->co_t == 0.0){
        s->countHG = 0;
        if(s->mzams >= 0.7)
            return 1; //high-mass main sequence
            else
                return 0; //low-mass main sequence
    }
    
    if((s->mHE_prev == 0.0 && s->he_t != 0.0) || s->countHG < 2 ){
        s->countHG++;
        return 2;
    }
    
    //I am not a WR, I have a Helium core, helium core growth rate has varied more than 50%
    if((s->mHE_dot > 0.0 && (s->mHE_dot_init - s->mHE_dot)/s->mHE_dot_init < 0.9 && s->co_t == 0.0)){
        s->countHG++;
        return max(2,current_type); //core He burning phase
    }
    
    //I am not a WR, I have a Helium core, helium core growth rate has varied more than 50%
    if(s->mHE_dot > 0.0 && (s->mHE_dot_init - s->mHE_dot)/s->mHE_dot_init > 0.9 && s->countHG >= 2 && s->co_t == 0.0){
        return 4; //core He burning phase
    }
    
    if(s->mHE_dot == 0.0 && s->he_t != 0.0 && s->co_t == 0.0)
        return 4;
    
    
    if(s->m_t != s->he_t && s->mCO_dot > 0.0){
        return 5; //Early-AGB 
    }
    
    if(s->m_t == s->he_t){ //looking for WR stars (naked helium)
        if(s->co_t == 0.0)
            return 7; //Naked helium main sequence
            else
                return 8; //Naked helium Hertzsprung gap
    }
    
    
    return -1;
    
}


char* Stellarevolution::phase_to_char(int int_phase){


	string name;

	if(int_phase == 0){
		name = "Low-mass main sequence";
	}
	else if (int_phase == 1){
		name = "High-mass main sequence";
	}
	else if (int_phase == 2){
		name = "Hertzsprung gap";
	}
	else if (int_phase == 3){
		name = "First giant branch";
	}
	else if (int_phase == 4){
		name = "Core He burning";
	}
	else if (int_phase == 5){
		name = "early AGB";
	}
	else if (int_phase == 6){
		name = "TP-AGB";
	}
	else if (int_phase == 7){
		name = "Naked helium MS";
	}
	else if (int_phase == 8){
		name = "Naked helium HG";
	}
	else if (int_phase == 9){
		name = "Naked helium giant";
	}
	else if (int_phase == 10){
		name = "Helium WD";
	}
	else if (int_phase == 11){
		name = "CO WD";
	}
	else if (int_phase == 12){
		name = "O/Ne WD";
	}
	else if (int_phase == 13){
		name = "Neutron star";
	}
	else if (int_phase == 14){
		name = "Black Hole";
	}
	else
        name = "Not A Star";
	
	
	return strdup(name.c_str());

}


int Stellarevolution::phase_to_int(char* char_phase){
    
	string local(char_phase);
    
	if(local == "Low-mass main sequence")
      return 0;
	else if (local == "High-mass main sequence")
		return 1;
    else if (local == "Hertzsprung gap")
		return 2;
	else if (local == "First giant branch")
      return 3;
	else if (local == "Core He burning")
        return 4;
   else if (local == "early AGB")
      return 5;
   else if (local == "TP-AGB")
      return 6;
   else if (local == "Naked helium MS")
      return 7;
   else if (local == "Naked helium HG")
      return 8;
   else if (local == "Naked helium giant")
      return 9;
   else if (local == "Helium WD")
      return 10;
   else if (local == "CO WD")
      return 11;
   else if (local == "O/Ne WD")
      return 12;
   else if (local == "Neutron star")
      return 13;
   else if (local == "Black Hole")
      return 14;
   else
      return -1;

}

void Stellarevolution::get_new_phase(star *s, int k1, int k2, int k3, int k4, int k5, int k6, int k7, int k8,
                                     float ttarg1, float ttarg2, float ttarg3, float ttarg4){
    
    float m1, q1, pha1_lower;
    
    if(s->wm.s[0] >= s->wm.s[1]){
        if(s->wz.s[0] >= s->wz.s[1]){
            //using track at zlow and m1
            m1 = (stevo_phase[k2] - stevo_phase[k1])/(stevo_times[k2] - stevo_times[k1]);
            q1 = stevo_phase[k2] - m1*stevo_times[k2];
            pha1_lower = m1*ttarg1 + q1;
            s->phase = phase_to_char((int) round(pha1_lower));
        }
        else if (s->wz.s[0] < s->wz.s[1]){
            //using track at zhigh and m1
            m1 = (stevo_phase[k6] - stevo_phase[k5])/(stevo_times[k6] - stevo_times[k5]);
            q1 = stevo_phase[k6] - m1*stevo_times[k6];
            pha1_lower = m1*ttarg3 + q1;
            s->phase = phase_to_char((int) round(pha1_lower));
            
        }
        
    }	
    
    else if(s->wm.s[0] < s->wm.s[1]){
        if(s->wz.s[0] < s->wz.s[1]){
            //using track at zhigh and m2
            m1 = (stevo_phase[k8] - stevo_phase[k7])/(stevo_times[k8] - stevo_times[k7]);
            q1 = stevo_phase[k8] - m1*stevo_times[k8];
            pha1_lower = m1*ttarg4 + q1;
            s->phase = phase_to_char((int) round(pha1_lower));
        }
        else if (s->wz.s[0] >= s->wz.s[1]){
            //using track at zlow and m2
            m1 = (stevo_phase[k4] - stevo_phase[k3])/(stevo_times[k4] - stevo_times[k3]);
            q1 = stevo_phase[k4] - m1*stevo_times[k4];
            pha1_lower = m1*ttarg2 + q1;
            s->phase = phase_to_char((int) round(pha1_lower));
            
            
        }
    }
    
    return;
    
}
                                     

void Stellarevolution::get_Ks(float life_percentage, float t0, float tf, int i0, int i1, int *k1, int *k2, float *ttarg){
 
    *k1 = *k1 = -1;
    
    *ttarg = t0 + life_percentage*(tf - t0);
    //I have no points in tables anymore.. cannot interpolate values.. just return and freeze all the values
    if(*ttarg > stevo_times[i1])
        return;
    
    *k2 = array_bound(&stevo_times[i0], i1-i0+1, *ttarg) + i0;
    *k1 = *k2-1;
    
    return;
    
}

float Stellarevolution::get_new_value(float t1, float m1, float tt1, float mm1, float ttarget1,
                           float t2, float m2, float tt2, float mm2, float ttarget2,
                           float t3, float m3, float tt3, float mm3, float ttarget3,
                           float t4, float m4, float tt4, float mm4, float ttarget4,
                           float w1, float w2, float z1, float z2){
    
    float slope = m1*t1;
    float intercept = mm1 - slope*tt1;
    float zlower1 = slope*ttarget1 + intercept;
    
    slope = m2*t2;
    intercept = mm2 - slope*tt2;
    float zlower2 = slope*ttarget2 + intercept;
    
    float V1 = w1 * zlower1 + w2 * zlower2;
    
    slope = m3*t3;
    intercept = mm3 - slope*tt3;
    float zupper1 = slope*ttarget3 + intercept;
    
    slope = m4*t4;
    intercept = mm4 - slope*tt4;
    float zupper2 = slope*ttarget4 + intercept;
    
    float V2 = w1 * zupper1 + w2 * zupper2;
    
    
    return (z1 * V1 + z2 * V2);
    
}
