#include <iostream>
#include <math.h>

#include <binary_dynamics.h>

using namespace std;


void binarydynamics::update_binary_properties(binary *bs){

   double sum_masses = bs->s1.m_t + bs->s2.m_t;
   double reduced_mass = (bs->s1.m_t*bs->s2.m_t)/sum_masses;

   double dx = bs->s2.pos.s[0] - bs->s1.pos.s[0];
   double dy = bs->s2.pos.s[1] - bs->s1.pos.s[1];
   double dz = bs->s2.pos.s[2] - bs->s1.pos.s[2];
   double distance = sqrt(dx*dx + dy*dy + dz*dz);

   double dvx = bs->s2.vel.s[0] - bs->s1.vel.s[0];
   double dvy = bs->s2.vel.s[1] - bs->s1.vel.s[1];
   double dvz = bs->s2.vel.s[2] - bs->s1.vel.s[2]; 
   double velocity = sqrt(dvx*dvx + dvy*dvy + dvz*dvz);

   double positive_specific_energy = sum_masses/distance - 0.5*velocity*velocity;

   bs->binding_energy = reduced_mass*positive_specific_energy; //in this way, binding energy is positive !!
	bs->a = (bs->s1.m_t*bs->s2.m_t)/(2.0*bs->binding_energy); //semimajor axis m1*m2/(2*E_bind)
	bs->period = 2.0*M_PI*sqrt((bs->a * bs->a * bs->a)/sum_masses); //period of the binary star

	double spec_energy = -positive_specific_energy;
	//specific angular momentum
   double Lx = dy*dvz - dz*dvy;
   double Ly = dz*dvx - dx*dvz;
   double Lz = dx*dvy - dy*dvx;

   double L = bs->ang_mom.s[3] = sqrt(Lx*Lx + Ly*Ly + Lz*Lz); //angular momentum absolute value
   bs->ang_mom.s[0] = Lx / L;
   bs->ang_mom.s[1] = Ly / L;
   bs->ang_mom.s[2] = Lz / L;   
   bs->ecc = sqrt(1.0 + (2.0*spec_energy*L*L)/(sum_masses*sum_masses)); //eccentricity of the binary system

	return;

}
