#include <iostream>
#include <math.h> 

#include <binary_functions.h>

using namespace std;



//define your own function HERE...
//DO NOT CHANGE OTHER FUNCTIONS
//TAKE MASS_TRANSFER FUNCTION AS EXAMPLE



void binaryfunction::mass_transfer(binary *bs, double systime){

	   
	cout<<" Performing MARIO mass transfer "<<endl;
	cout<<" System time = "<<systime<<endl;
   double amount = 5.0;

   bs->s2.m_t = bs->s2.m_t - amount;
   bs->s1.m_t = bs->s1.m_t + amount;


   //check if one star is WR
   //(the star that loses mass)
   //adjust mass transfer


   if(bs->s2.m_t < bs->s2.he_t) //I have become a WR star
      bs->s2.he_t = bs->s2.m_t;


	return;

}




float binaryfunction::wind_accretion(float m1, float m2, float r1, float dmdt, binary *bs){

	float maccr = 0.0;

	float DM = dt*dmdt; //mass loss of primary due to winds during timestep

   //square of binary eccentricity
   float ecc2 = bs->ecc * bs->ecc;//MM: bs->ecc calls eccentricity

   //square of wind  velocity from hurley2002 eq.9
   float vw2 = (2.0 * betaw * m1)/r1; //eq 9 of hurley 2002, MM: s1.r should contain the radius of star 1   

   //square of binary orbital velocity
   float vorb2 = (m1+m2)/bs->a;//MM: bs->a calls semi-major axis


   /*begin eq. 6 of Hurley 2002 */
   maccr = -1.0/sqrt(1.0 - ecc2);
   maccr *= (m2*m2)/(vw2*vw2);
   maccr *= aw/(2.0*bs->a*bs->a); //MM: bs->a calls semi-major axis

   float vel_norm = sqrt(1.0 + (vorb2/vw2));
   maccr *= 1.0/(vel_norm*vel_norm*vel_norm);

   maccr *= DM;
   /*end eq. 6 of Hurley 2002 */
   maccr = min(maccr, 0.8f*DM); //hurley2002 eq10



	return maccr;
}



void binaryfunction::angmom_variation(binary *bs, float m1, float m2, float dmdt, float maccr){

	float djorb = 0.0;

   float oorb = 2.0 * M_PI/bs->period; //MM bs->period should contain orbital period in correct units;
	float DM = dt*dmdt; //mass loss due to winds during timestep
	float Mtot = m1+m2; //total mass of the binary system
	float ecc2 = bs->ecc * bs->ecc;//MM: bs->ecc calls eccentricity

   //begin eq 18 hurley
   djorb = DM * m2 * m2;
   djorb -= maccr * m1 * m2;
   djorb *= (bs->a*bs->a) * sqrt(1.-ecc2) * oorb/(Mtot*Mtot);
   //end eq 18 hurley 2002
	

   //calculate ecc change due to wind mass loss
////   float delet = -bs->ecc * maccr * (0.5/m2 +1.0/Mtot); //eq 15 hurley 2002

   /* from now on lines 674-684 of evolv2.f in BSE*/
   /* AAT: same update of eccentricity exist in tidal_dissipation, should be joined eventually 
    * copying from angmom_variation. should be joined someday 
   float ecc1 = bs->ecc - delet*dt;
   ecc1 = max(ecc1, 0.0);

   if(ecc1<1.0e-10){
     ecc1 = 0.0;
   }

   if(ecc1>1.0) {
     cout<<" Binary has broken; pls do something!!! "<<endl;
   }*/


   /*updating eccentricity after wind mass transfer*/
//   bs->ecc = ecc1;

   /*updating orb ang mom  after wind mass transfer*/
   bs->ang_mom.s[3] = bs->ang_mom.s[3] - djorb*dt; //MM: bs should contain ang mom jorb we should define dt


   ecc2 = bs->ecc*bs->ecc;

   /*updating semi-major axis after wind mass transfer*/
   bs->a = (bs->ang_mom.s[3]*bs->ang_mom.s[3])*Mtot/((1.0-ecc2)*(2.0*M_PI)*(2.0*M_PI)*m1*m1*m2*m2); //MM check the units 

   /*updating period after wind mass transfer*/
   bs->period = bs->a*sqrt(bs->a/(Mtot));
   /* end lines 674-684 of evolv2.f in BSE*/


	return;

}

void binaryfunction::wind_mass_transfer(binary *bs, double systime){ //MM I think bs should contain the orbital properties of the binary systems, including semi-major axis a and eccentricity ecc !!!


  /* Calculate how much of wind mass loss from companion will be
     accreted (Boffin & Jorissen, A&A 1988, 205, 155).
     from hurley 2002 sec 2.1 */
	   
	cout<<" Performing wind mass transfer "<<endl;
	cout<<" System time = "<<systime<<endl;

	betaw = 1.0; //coefficient in hurley2002 eq.9 - MM: a function should be added to calculate it
	aw = 1.5; //coefficient in hurley2002 for bondi-hoyle accretion (constant)


	dt = bs->dt;
	bs->s1.mwind_accr = bs->s2.mwind_accr = 0.0; //initialize accreted masses

	//perform mass transfer from star s1 to star s2
	maccr_1_to_2 = wind_accretion(bs->s1.m_t, bs->s2.m_t, bs->s1.r_t, bs->s1.m_dot, bs);
	//perform mass transfer from star s2 to star s1
	maccr_2_to_1 = wind_accretion(bs->s2.m_t, bs->s1.m_t, bs->s2.r_t, bs->s2.m_dot, bs);


	//update accreted masses
   bs->s2.mwind_accr = maccr_1_to_2;
	bs->s1.mwind_accr = maccr_2_to_1;


	//MS: I think 
	//update semimajor axis, period, angular momentum and eccentricity of the binary system
	angmom_variation(bs, bs->s1.m_t, bs->s2.m_t, bs->s1.m_dot, bs->s2.mwind_accr);


	/*update masses (MM I think it is the last one but I am not sure)*/
	bs->s2.m_t = bs->s2.m_t + bs->s2.mwind_accr;
	bs->s1.m_t = bs->s1.m_t + bs->s1.mwind_accr;

	return;

}

void binaryfunction::tidal_dissipation(binary *bs, double ospin, double dt){
  
	/*AAT: s1: star undergoing tides, s2 companion 
	  ospin: Omega_spin, to be computed from 7.2 Hurley00 WIP
	Calculate the tidal dissipation (Hurley 2002, 2000) */
	
	//AAT: to be computed, donno how
	double lum = 0.0; // luminosity
	double menv = 0.0; // envelope mass
	double renv = 0.0; // envelope radius
	
	double ptid = fabs(1./bs->period - ospin/2.0/M_PI); // Hurley02 eq 33
	double tauconv = 0.4311 * pow((menv * renv * (bs->s1.r_t - 0.5 * renv)/(3*lum)), 1./3.); // Hurley02 eq 31
	double frac = 0.5 * ptid / tauconv;
	double fconv = min(1.0, frac*frac); // Hurley02 eq 32
	double ktc = 2./21 * fconv/tauconv * menv/bs->s1.m_t; // Hurley02 eq 30
	
	double q = bs->s2.m_t / bs->s1.m_t;
	double ecc2 = bs->ecc * bs->ecc;
   double sqome2 = sqrt(1.0 - ecc2);
	double ra = bs->s1.r_t/bs->a;
	double f3 = 1.0+ecc2*(3.75+ecc2*(1.875+ecc2*7.8125e-02));
	double f5 = 1.0+ecc2*(3.+ecc2*0.375);
	
	double delet = 27 * ktc * q * (q+1) * pow(ra, 8) * bs->ecc/pow(sqome2, 13) ; // lines 526 evolv2.f
	delet *= f3 - (11./18.)*pow(sqome2, 3) *f5* ospin/(2.0*M_PI/bs->period);
	
	/* AAT: copying from angmom_variation. should be joined someday 
	 * from now on lines 674-684 of evolv2.f in BSE*/
	float ecc1 = bs->ecc - delet*dt;
	ecc1 = max(ecc1, 0.0f);

	if(ecc1<1.0e-10){
	ecc1 = 0.0;
	}

	if(ecc1>1.0) {
	cout<<" Binary has broken; pls do something!!! "<<endl;
	}
	
	return;

}
