#include <utilities.h>
#include <errhand.h>
#include <cmath>

using namespace std;


float utilities::Roche_lobe(float q, float a, float *RL_over_a){
    
    double p = pow(q, 0.33333333333);
    *RL_over_a = (0.49*p*p) / (0.6*p*p + log(1.0 + p));
    
    return ( a * (*RL_over_a) );
}

bool utilities::string_comp(string s1, string s2){

	if (s1.find(s2) != string::npos)
      return true;
   else
      return false;
}

int utilities::array_bound(float *array, int size, float val){
   int lb = 0; 
	int hb = size-1;

   while (lb < hb) {
		int mid = lb + (hb - lb)/2;

      if (array[mid] < val)
         lb = mid + 1;
      else
         hb = mid;
   }
   return lb;
}


float utilities::SplitArray(vector<float> &array, vector<int>& ind, float pivot, int startIndex, int endIndex){

    int leftBoundary = startIndex;
    int rightBoundary = endIndex;

    while(leftBoundary < rightBoundary)             //shuttle pivot until the boundaries meet
    {
         while( pivot < array[rightBoundary]       //keep moving until a lesser element is found
                && rightBoundary > leftBoundary)   //or until the leftBoundary is reached
         {
              rightBoundary--;                      //move left
         }

         swap(array[leftBoundary], array[rightBoundary]);
         swap(ind[leftBoundary], ind[rightBoundary]);


         while( pivot >= array[leftBoundary]       //keep moving until a greater or equal element is found
                && leftBoundary < rightBoundary)   //or until the rightBoundary is reached
         {
              leftBoundary++;                        //move right
         }
         swap(array[rightBoundary], array[leftBoundary]);
         swap(ind[rightBoundary], ind[leftBoundary]);

    }

    return leftBoundary;                              //leftBoundary is the split point because

}


float utilities::SplitArray(vector<float> &array, vector<cl_int2>& ind, float pivot, int startIndex, int endIndex){

    int leftBoundary = startIndex;
    int rightBoundary = endIndex;

    while(leftBoundary < rightBoundary)             //shuttle pivot until the boundaries meet
    {
         while( pivot < array[rightBoundary]       //keep moving until a lesser element is found
                && rightBoundary > leftBoundary)   //or until the leftBoundary is reached
         {
              rightBoundary--;                      //move left
         }

         swap(array[leftBoundary], array[rightBoundary]);
         swap(ind[leftBoundary].s[0], ind[rightBoundary].s[0]);
			swap(ind[leftBoundary].s[1], ind[rightBoundary].s[1]);


         while( pivot >= array[leftBoundary]       //keep moving until a greater or equal element is found
                && leftBoundary < rightBoundary)   //or until the rightBoundary is reached
         {
              leftBoundary++;                        //move right
         }
         swap(array[rightBoundary], array[leftBoundary]);
         swap(ind[rightBoundary].s[0], ind[leftBoundary].s[0]);
			swap(ind[rightBoundary].s[1], ind[leftBoundary].s[1]);

    }

    return leftBoundary;                              //leftBoundary is the split point because

}


float utilities::SplitArray(vector<float> &array, vector<string>& ind, float pivot, int startIndex, int endIndex){

    int leftBoundary = startIndex;
    int rightBoundary = endIndex;

    while(leftBoundary < rightBoundary)             //shuttle pivot until the boundaries meet
    {
         while( pivot < array[rightBoundary]       //keep moving until a lesser element is found
                && rightBoundary > leftBoundary)   //or until the leftBoundary is reached
         {
              rightBoundary--;                      //move left
         }

         swap(array[leftBoundary], array[rightBoundary]);
         swap(ind[leftBoundary], ind[rightBoundary]);


         while( pivot >= array[leftBoundary]       //keep moving until a greater or equal element is found
                && leftBoundary < rightBoundary)   //or until the rightBoundary is reached
         {
              leftBoundary++;                        //move right
         }
         swap(array[rightBoundary], array[leftBoundary]);
         swap(ind[rightBoundary], ind[leftBoundary]);

    }

    return leftBoundary;                              //leftBoundary is the split point because

}


void utilities::Quick_Sort(vector<float> &array, vector<string>& ind, int startIndex, int endIndex){

 float pivot = array[startIndex];                  //pivot element is the leftmost element
   int splitPoint;

    if(endIndex > startIndex)                         //if they are equal, it means there is
    {
        splitPoint = SplitArray(array, ind, pivot, startIndex, endIndex);
        array[splitPoint] = pivot;
        Quick_Sort(array, ind, startIndex, splitPoint-1);   //Quick sort first half
        Quick_Sort(array, ind, splitPoint+1, endIndex);    //Quick sort second half
    }

}


void utilities::Quick_Sort(vector<float> &array, vector<cl_int2>& ind, int startIndex, int endIndex){

 float pivot = array[startIndex];                  //pivot element is the leftmost element
   int splitPoint;

    if(endIndex > startIndex)                         //if they are equal, it means there is
    {
        splitPoint = SplitArray(array, ind, pivot, startIndex, endIndex);
        array[splitPoint] = pivot;
        Quick_Sort(array, ind, startIndex, splitPoint-1);   //Quick sort first half
        Quick_Sort(array, ind, splitPoint+1, endIndex);    //Quick sort second half
    }

}


void utilities::Quick_Sort(vector<float> &array, vector<int>& ind, int startIndex, int endIndex){

 float pivot = array[startIndex];                  //pivot element is the leftmost element
   int splitPoint;

    if(endIndex > startIndex)                         //if they are equal, it means there is
    {
        splitPoint = SplitArray(array, ind, pivot, startIndex, endIndex);
        array[splitPoint] = pivot;
        Quick_Sort(array, ind, startIndex, splitPoint-1);   //Quick sort first half
        Quick_Sort(array, ind, splitPoint+1, endIndex);    //Quick sort second half
    }

}

