#include <iostream>

#include <Stellarevolution.h>
#include <errhand.h>

#include <iomanip>

using namespace std;


int main(){

	//class that contains all stevo functions
	Stellarevolution stevo;

	//a generic single star
	star s;


	//input parameters
	s.mzams = 12.0; //ZAMS mass
	s.z = 0.02; //metallicity
	s.sn = (char*)"delayed"; //SN explosion model (can be: delayed, rapid, startrack)

	//integration parameters
	s.time_step = 0.001; //time step in Myr
	s.t_loc = 0.0; //time for initialization
	
	
	double integration_time = 100.0; //total integration time in Myr
	double system_time = 0.0; //global time
	

	//ofstream for printing out results (mass as a function of time)
	ofstream out;
	out.open("output.dat", ios::out);
	out<<scientific<<setprecision(10); //10 digits

	try{ // HANDLE EXCEPTIONS


		//load tables and load fixed arrays
		//call it always here... general initialization 
		stevo.load_stevo_arrays();

		//get all the needed parameters for the star
		stevo.get_final_parameters(&s);


		//initialize stars at specific times!! **0.01 is time zero in our tables**
		stevo.stevoevolve(&s); 

		//just a counter
		int count = 0;
        bool done = false;

		//stellar evolution MAIN cycle... evolve stars following stellar evolution tables
		for(;;){
            count++;
			//advance local time for both stars using the time step
			//total time... keep track of total time in Myr
			system_time = count*s.time_step;



			//evolve the first star as it was isolated
			//it evolves star s at time s.t_loc = s.t_loc + s.time_step
			stevo.stevoevolve(&s);


            //print out some results (SOLAR UNITS)
			out<<system_time<<"   "<<s.m_t<<"   "<<s.he_t<<"   "<<s.co_t<<"   "<<s.r_t<<"   "<<s.l_t<<"   "<<s.phase<<endl;

			//break condition
			if(system_time > integration_time)
				break;

		}//END MAIN CYCLE


	}
	catch (errhand &t){}


	//close output file
	out.close();


	return 0;

}
